#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <locale.h>
#include <time.h>
#include "linear.h"
#include "tron.h"
#include <mpi.h>
#include <set>
#include <map>

#ifndef TIMER
#define TIMER
#define NS_PER_SEC 1000000000
double wall_time_diff(int64_t ed, int64_t st)
{
	return (double)(ed-st)/(double)NS_PER_SEC;
}

int64_t wall_clock_ns()
{
#if __unix__
	struct timespec tspec;
	clock_gettime(CLOCK_MONOTONIC, &tspec);
	return tspec.tv_sec*NS_PER_SEC + tspec.tv_nsec;
#else
#if __MACH__
	return 0;
#else
	struct timeval tv;
	gettimeofday( &tv, NULL );
	return tv.tv_sec*NS_PER_SEC + tv.tv_usec*1000;
#endif
#endif
}
#endif

int liblinear_version = LIBLINEAR_VERSION;
typedef signed char schar;
template <class T> static inline void swap(T& x, T& y) { T t=x; x=y; y=t; }
template <class T> static inline void swap(T* a, int x, int y) { T t=a[x]; a[x]=a[y]; a[y]=t; }
#ifndef min
template <class T> static inline T min(T x,T y) { return (x<y)?x:y; }
#endif
#ifndef max
template <class T> static inline T max(T x,T y) { return (x>y)?x:y; }
#endif
template <class S, class T> static inline void clone(T*& dst, S* src, int n)
{
	dst = new T[n];
	memcpy((void *)dst,(void *)src,sizeof(T)*n);
}
#define Malloc(type,n) (type *)malloc((n)*sizeof(type))
#define INF HUGE_VAL

static void print_string_stdout(const char *s)
{
	fputs(s,stdout);
	fflush(stdout);
}
static void print_null(const char *s) {}

static void (*liblinear_print_string) (const char *) = &print_string_stdout;

#if 1
static void info(const char *fmt,...)
{
	if(mpi_get_rank()!=0)
		return;
	char buf[BUFSIZ];
	va_list ap;
	va_start(ap,fmt);
	vsprintf(buf,fmt,ap);
	va_end(ap);
	(*liblinear_print_string)(buf);
}
#else
static void info(const char *fmt,...) {}
#endif
class sparse_operator
{
public:
	static double nrm2_sq(const feature_node *x)
	{
		double ret = 0;
		while(x->index != -1)
		{
			ret += x->value*x->value;
			x++;
		}
		return (ret);
	}

	static double dot(const double *s, const feature_node *x)
	{
		double ret = 0;
		while(x->index != -1)
		{
			ret += s[x->index-1]*x->value;
			x++;
		}
		return (ret);
	}

	static void axpy(const double a, const feature_node *x, double *y)
	{
		while(x->index != -1)
		{
			y[x->index-1] += a*x->value;
			x++;
		}
	}
};

class l2r_lr_fun: public function
{
public:
	l2r_lr_fun(const problem *prob, double *C);
	~l2r_lr_fun();

	double fun(double *w);
	void grad(double *w, double *g);
	void Hv(double *s, double *Hs);

	int get_nr_variable(void);

private:
	void Xv(double *v, double *Xv);
	void XTv(double *v, double *XTv);

	double *C;
	double *z;
	double *D;
	const problem *prob;
};

l2r_lr_fun::l2r_lr_fun(const problem *prob, double *C)
{
	int l=prob->l;

	this->prob = prob;

	z = new double[l];
	D = new double[l];
	this->C = C;
}

l2r_lr_fun::~l2r_lr_fun()
{
	delete[] z;
	delete[] D;
}


double l2r_lr_fun::fun(double *w)
{
	int i;
	double f=0;
	double *y=prob->y;
	int l=prob->l;
	int w_size=get_nr_variable();

	Xv(w, z);

	for(i=0;i<w_size;i++)
		f += w[i]*w[i];
	f /= 2.0;
	for(i=0;i<l;i++)
	{
		double yz = y[i]*z[i];
		if (yz >= 0)
			f += C[i]*log(1 + exp(-yz));
		else
			f += C[i]*(-yz+log(1 + exp(yz)));
	}

	return(f);
}

void l2r_lr_fun::grad(double *w, double *g)
{
	int i;
	double *y=prob->y;
	int l=prob->l;
	int w_size=get_nr_variable();

	for(i=0;i<l;i++)
	{
		z[i] = 1/(1 + exp(-y[i]*z[i]));
		D[i] = z[i]*(1-z[i]);
		z[i] = C[i]*(z[i]-1)*y[i];
	}
	XTv(z, g);

	for(i=0;i<w_size;i++)
		g[i] = w[i] + g[i];
}

int l2r_lr_fun::get_nr_variable(void)
{
	return prob->n;
}

void l2r_lr_fun::Hv(double *s, double *Hs)
{
	int i;
	int l=prob->l;
	int w_size=get_nr_variable();
	feature_node **x=prob->x;

	for(i=0;i<w_size;i++)
		Hs[i] = 0;
	for(i=0;i<l;i++)
	{
		feature_node * const xi=x[i];
		double xTs = sparse_operator::dot(s, xi);

		xTs = C[i]*D[i]*xTs;

		sparse_operator::axpy(xTs, xi, Hs);
	}
	for(i=0;i<w_size;i++)
		Hs[i] = s[i] + Hs[i];
}

void l2r_lr_fun::Xv(double *v, double *Xv)
{
	int i;
	int l=prob->l;
	feature_node **x=prob->x;

	for(i=0;i<l;i++)
		Xv[i]=sparse_operator::dot(v, x[i]);
}

void l2r_lr_fun::XTv(double *v, double *XTv)
{
	int i;
	int l=prob->l;
	int w_size=get_nr_variable();
	feature_node **x=prob->x;

	for(i=0;i<w_size;i++)
		XTv[i]=0;
	for(i=0;i<l;i++)
		sparse_operator::axpy(v[i], x[i], XTv);
}

class l2r_l2_svc_fun: public function
{
public:
	l2r_l2_svc_fun(const problem *prob, double *C);
	~l2r_l2_svc_fun();

	double fun(double *w);
	void grad(double *w, double *g);
	void Hv(double *s, double *Hs);

	int get_nr_variable(void);

protected:
	void Xv(double *v, double *Xv);
	void subXTv(double *v, double *XTv);

	double *C;
	double *z;
	int *I;
	int sizeI;
	const problem *prob;
};

l2r_l2_svc_fun::l2r_l2_svc_fun(const problem *prob, double *C)
{
	int l=prob->l;

	this->prob = prob;

	z = new double[l];
	I = new int[l];
	this->C = C;
}

l2r_l2_svc_fun::~l2r_l2_svc_fun()
{
	delete[] z;
	delete[] I;
}

double l2r_l2_svc_fun::fun(double *w)
{
	int i;
	double f=0;
	double *y=prob->y;
	int l=prob->l;
	int w_size=get_nr_variable();

	Xv(w, z);

	for(i=0;i<w_size;i++)
		f += w[i]*w[i];
	f /= 2.0;
	for(i=0;i<l;i++)
	{
		z[i] = y[i]*z[i];
		double d = 1-z[i];
		if (d > 0)
			f += C[i]*d*d;
	}

	return(f);
}

void l2r_l2_svc_fun::grad(double *w, double *g)
{
	int i;
	double *y=prob->y;
	int l=prob->l;
	int w_size=get_nr_variable();

	sizeI = 0;
	for (i=0;i<l;i++)
		if (z[i] < 1)
		{
			z[sizeI] = C[i]*y[i]*(z[i]-1);
			I[sizeI] = i;
			sizeI++;
		}
	subXTv(z, g);

	for(i=0;i<w_size;i++)
		g[i] = w[i] + 2*g[i];
}

int l2r_l2_svc_fun::get_nr_variable(void)
{
	return prob->n;
}

void l2r_l2_svc_fun::Hv(double *s, double *Hs)
{
	int i;
	int w_size=get_nr_variable();
	feature_node **x=prob->x;

	for(i=0;i<w_size;i++)
		Hs[i]=0;
	for(i=0;i<sizeI;i++)
	{
		feature_node * const xi=x[I[i]];
		double xTs = sparse_operator::dot(s, xi);

		xTs = C[I[i]]*xTs;

		sparse_operator::axpy(xTs, xi, Hs);
	}
	for(i=0;i<w_size;i++)
		Hs[i] = s[i] + 2*Hs[i];
}

void l2r_l2_svc_fun::Xv(double *v, double *Xv)
{
	int i;
	int l=prob->l;
	feature_node **x=prob->x;

	for(i=0;i<l;i++)
		Xv[i]=sparse_operator::dot(v, x[i]);
}

void l2r_l2_svc_fun::subXTv(double *v, double *XTv)
{
	int i;
	int w_size=get_nr_variable();
	feature_node **x=prob->x;

	for(i=0;i<w_size;i++)
		XTv[i]=0;
	for(i=0;i<sizeI;i++)
		sparse_operator::axpy(v[i], x[I[i]], XTv);
}

class l2r_l2_svr_fun: public l2r_l2_svc_fun
{
public:
	l2r_l2_svr_fun(const problem *prob, double *C, double p);

	double fun(double *w);
	void grad(double *w, double *g);

private:
	double p;
};

l2r_l2_svr_fun::l2r_l2_svr_fun(const problem *prob, double *C, double p):
	l2r_l2_svc_fun(prob, C)
{
	this->p = p;
}

double l2r_l2_svr_fun::fun(double *w)
{
	int i;
	double f=0;
	double *y=prob->y;
	int l=prob->l;
	int w_size=get_nr_variable();
	double d;

	Xv(w, z);

	for(i=0;i<w_size;i++)
		f += w[i]*w[i];
	f /= 2;
	for(i=0;i<l;i++)
	{
		d = z[i] - y[i];
		if(d < -p)
			f += C[i]*(d+p)*(d+p);
		else if(d > p)
			f += C[i]*(d-p)*(d-p);
	}

	return(f);
}

void l2r_l2_svr_fun::grad(double *w, double *g)
{
	int i;
	double *y=prob->y;
	int l=prob->l;
	int w_size=get_nr_variable();
	double d;

	sizeI = 0;
	for(i=0;i<l;i++)
	{
		d = z[i] - y[i];

		// generate index set I
		if(d < -p)
		{
			z[sizeI] = C[i]*(d+p);
			I[sizeI] = i;
			sizeI++;
		}
		else if(d > p)
		{
			z[sizeI] = C[i]*(d-p);
			I[sizeI] = i;
			sizeI++;
		}

	}
	subXTv(z, g);

	for(i=0;i<w_size;i++)
		g[i] = w[i] + 2*g[i];
}

class l1r_lr_fun
{
public:
	l1r_lr_fun(const problem *prob, double *C);
	virtual ~l1r_lr_fun();

	double fun(double *w);
	void loss_grad(double *w, double *g);
	void cal_pg(double *w, double *loss_g, double *pg);
	int get_nr_variable(void);
	int get_nr_data(void);
	void cal_sqrtCD(double *sqrtCD);
	double line_search(double *step, double *old_w, double *loss_g, double *pg, double *step_size, double eta,  double old_f, int *num_line_search_steps, double *w);
	//	void get_Xw(double *Xw_);
	double *get_Xw();
	virtual void get_diag(double *M) = 0;
	virtual void Xv(double *v, double *Xv) = 0;
	void Hv(double *s, double *Hs);
	
protected:
	virtual void XTv(double *v, double *XTv) = 0;

	double *C;
	double *z;
	double *Xw;
	double *D;
	const problem *prob;
	double reg;
	double current_f;
	int start;
	int length;
};

l1r_lr_fun::l1r_lr_fun(const problem *prob, double *C)
{
	int l=prob->l;

	this->prob = prob;

	z = new double[l];
	Xw = new double[l];
	D = new double[l];
	this->C = C;
	int w_size = get_nr_variable();
	int nr_node = mpi_get_size();
	int rank = mpi_get_rank();
	int shift = int(ceil(double(w_size) / double(nr_node)));
	this->start = shift * rank;
	this->length = min(max(w_size - start, 0), shift);
	if (length == 0)
		start = 0;
}

l1r_lr_fun::~l1r_lr_fun()
{
	delete[] z;
	delete[] Xw;
	delete[] D;
}

double l1r_lr_fun::fun(double *w)
{
	int i;
	double f=0;
	double *y=prob->y;
	int l=prob->l;

	reg = 0;

	Xv(w, Xw);

	for(i=start;i<start + length;i++)
		reg += fabs(w[i]);
	for(i=0;i<l;i++)
	{
		double yXw = y[i]*Xw[i];
		if (yXw >= 0)
			f += C[i]*log(1 + exp(-yXw));
		else
			f += C[i]*(-yXw+log(1 + exp(yXw)));
	}
	double buffer[2] = {f, reg};
	mpi_allreduce(buffer, 2, MPI_DOUBLE, MPI_SUM);
	f = buffer[0];
	reg = buffer[1];
	f += reg;

	current_f = f;
	return(f);
}

double *l1r_lr_fun::get_Xw()
{
	return Xw;
}

void l1r_lr_fun::loss_grad(double *w, double *g)
{
	int i;
	double *y=prob->y;
	int l=prob->l;
	int n = prob->n;

	for(i=0;i<l;i++)
	{
		z[i] = 1/(1 + exp(-y[i]*Xw[i]));
		D[i] = z[i]*(1-z[i]);
		z[i] = C[i]*(z[i]-1)*y[i];
	}
	XTv(z, g);
	mpi_allreduce(g, n, MPI_DOUBLE, MPI_SUM);
}


void l1r_lr_fun::cal_pg(double *w, double *loss_g, double *pg)
{
	int n = prob->n;
	int i;

	for (i=0; i<n; i++)
		if (w[i] > 0)
			pg[i] = 1 + loss_g[i];
		else if (w[i] < 0)
			pg[i] = -1 + loss_g[i];
		else
		{
			if (1 + loss_g[i] < 0)
				pg[i] = 1 + loss_g[i];
			else if (-1 + loss_g[i] > 0)
				pg[i] = -1 + loss_g[i];
			else
				pg[i] = 0;
		}
}


int l1r_lr_fun::get_nr_variable(void)
{
	return prob->n;
}


int l1r_lr_fun::get_nr_data(void)
{
	return prob->l;
}

void l1r_lr_fun::cal_sqrtCD(double *sqrtCD)
{
	int l=prob->l;
	for (int i=0;i<l;i++)
		sqrtCD[i] = sqrt(C[i]*D[i]);
}

double l1r_lr_fun::line_search(double *step, double *old_w, double *loss_g, double *pg, double *step_size, double eta,  double old_f, int *num_line_search_steps, double *w)
{
	int i;
	int w_size = get_nr_variable();
	int max_num_linesearch = 1000;
	double f;

	int num_linesearch;

	for(num_linesearch=1; num_linesearch <= max_num_linesearch; num_linesearch++)
	{
		for (i=0; i<w_size; i++)
		{
			w[i] = old_w[i] + (*step_size)*step[i];
			if (old_w[i] > 0)
				w[i] = max(w[i], 0.0);
			else if (old_w[i] < 0)
				w[i] = min(w[i], 0.0);
			else
			{
				if (-(1 + loss_g[i]) > 0)
					w[i] = max(w[i], 0.0);
				else if (-(-1 + loss_g[i]) < 0)
					w[i] = min(w[i], 0.0);
				else
					w[i] = 0;
			}
		}

		f = fun(w);
		double tmp = 0;
		for (i=start; i<start + length; i++)
			tmp += pg[i]*(w[i]-old_w[i]);
		mpi_allreduce(&tmp, 1, MPI_DOUBLE, MPI_SUM);

		if (f - old_f - eta*tmp <= 0)
			break;
		(*step_size) *= 0.5;
	}
	info("step size %g\n", (*step_size));

	if (num_linesearch >= max_num_linesearch)
		return 0;

	*num_line_search_steps = num_linesearch;

	return f;
}

void l1r_lr_fun::Hv(double *s, double *Hs)
{
	int i;
	int l=prob->l;
	int w_size=get_nr_variable();
	double *wa = new double[l];

	Xv(s, wa);
	for(i=0;i<l;i++)
		wa[i] = C[i]*D[i]*wa[i];

	XTv(wa, Hs);
	mpi_allreduce(Hs, w_size, MPI_DOUBLE, MPI_SUM);
	delete[] wa;
}

class l1r_lr_fun_row: public l1r_lr_fun
{
public:
	l1r_lr_fun_row(const problem *prob_col, double *C);
	~l1r_lr_fun_row();

	void get_diag(double *M);
	void Xv(double *v, double *Xv);

private:
	void XTv(double *v, double *XTv);
};


l1r_lr_fun_row::l1r_lr_fun_row(const problem *prob_col, double *C):
	l1r_lr_fun(prob_col, C)
{
}

l1r_lr_fun_row::~l1r_lr_fun_row()
{
}


void l1r_lr_fun_row::get_diag(double *M)
{
	int i;
	int l = prob->l;
	int w_size=get_nr_variable();
	feature_node **x = prob->x;

	for(i = 0; i < w_size; i++)
		M[i] = 0;

	for(i = 0; i < l; i++)
	{
		feature_node *s = x[i];
		while(s->index!=-1)
		{
			M[s->index-1] += s->value*s->value*C[i]*D[i];
			s++;
		}
	}
	mpi_allreduce(M, w_size, MPI_DOUBLE, MPI_SUM);
}

void l1r_lr_fun_row::Xv(double *v, double *Xv)
{

	int i;
	int l=prob->l;
	feature_node **x=prob->x;

	for(i=0;i<l;i++)
	{
		feature_node *s=x[i];
		Xv[i]=0;
		while(s->index!=-1)
		{
			Xv[i]+=v[s->index-1]*s->value;
			s++;
		}
	}
}

void l1r_lr_fun_row::XTv(double *v, double *XTv)
{
	int i;
	int l=prob->l;
	int w_size=get_nr_variable();
	feature_node **x=prob->x;

	for(i=0;i<w_size;i++)
		XTv[i]=0;
	for(i=0;i<l;i++)
	{
		feature_node *s=x[i];
		while(s->index!=-1)
		{
			XTv[s->index-1]+=v[i]*s->value;
			s++;
		}
	}

}


class l1r_lr_fun_col: public l1r_lr_fun
{
public:
	l1r_lr_fun_col(const problem *prob_col, double *C);
	~l1r_lr_fun_col();

	void get_diag(double *M);
	void Xv(double *v, double *Xv);

private:
	void XTv(double *v, double *XTv);
};


l1r_lr_fun_col::l1r_lr_fun_col(const problem *prob_col, double *C):
	l1r_lr_fun(prob_col, C)
{
}

l1r_lr_fun_col::~l1r_lr_fun_col()
{
}

void l1r_lr_fun_col::get_diag(double *M)
{
	int j;
	int w_size=get_nr_variable();
	feature_node **x = prob->x;

	for(j = 0; j < w_size; j++)
	{
		M[j] = 0;

		feature_node *s = x[j];
		while(s->index!=-1)
		{
			M[j] += s->value*s->value*C[s->index-1]*D[s->index-1];
			s++;
		}
	}
	mpi_allreduce(M, w_size, MPI_DOUBLE, MPI_SUM);
}

void l1r_lr_fun_col::Xv(double *v, double *Xv)
{
	int i;
	int l=prob->l;
	int w_size=get_nr_variable();
	feature_node **x=prob->x;
	//	int nnz = 0;
	//	int w_nnz = 0;

	for(i=0;i<l;i++)
		Xv[i]=0;
	for(i=0;i<w_size;i++)
	{
		if (v[i] != 0)
		{
			//			w_nnz++;
			feature_node *s=x[i];
			while(s->index!=-1)
			{
				Xv[s->index-1]+=v[i]*s->value;
				s++;
				//				nnz++;
			}
		}
	}
	//	info("w_nnz %d nnz %d\n", w_nnz, nnz);
}

void l1r_lr_fun_col::XTv(double *v, double *XTv)
{

	int i;
	int w_size=get_nr_variable();
	feature_node **x=prob->x;

	for(i=0;i<w_size;i++)
	{
		feature_node *s=x[i];
		XTv[i]=0;
		while(s->index!=-1)
		{
			XTv[i]+=v[s->index-1]*s->value;
			s++;
		}
	}
}

// begin of l1 solvers

class OWLQN
{
public:
	OWLQN(const l1r_lr_fun *fun_obj, double eps = 0.1, int m=10, double eta = 1e-4, int max_iter = 5000);
	~OWLQN();

	void owlqn(double *w, int solver_type_ = 0);
	void set_print_string(void (*i_print) (const char *buf));

private:
	void update_inner_products(double** inner_product_matrix, int k, int DynamicM, double* s, double* y, double* pg);
	void TwoLoop(double** inner_product_matrix, double* rho, int DynamicM, int k, double* delta);
	void TwoLoop(double* g, double* s, double* y, double* rho, int DynamicM, double *step, int k, int n);
	double eps;
	double eta;
	int max_iter;
	int M;
	l1r_lr_fun *fun_obj;
	void info(const char *fmt,...);
	void (*owlqn_print_string)(const char *buf);
	int start;
	int length;
	int *recv_count;
	int *displace;
};


class L1_LCOMM
{
public:
	L1_LCOMM(const l1r_lr_fun *fun_obj, double eps = 0.1, int m=10, double eta = 1e-4, int max_iter = 5000);
	~L1_LCOMM();

	void lcomm(double *w, int solver_type_);
	void set_print_string(void (*i_print) (const char *buf));
private:
	void compute_system(int iter, int DynamicM, int l, double* XP, double* sqrtCD, double* system);
	double eps;
	double eta;
	int max_iter;
	int M;
	l1r_lr_fun *fun_obj;
	void info(const char *fmt,...);
	void (*lcomm_print_string)(const char *buf);
	int length;
	int start;
	int *recv_count;
	int *displace;
	
};

class L1_NEWTON
{
public:
	L1_NEWTON(const l1r_lr_fun *fun_obj, double eps = 0.1, int m=10, double eta = 1e-4, int max_iter = 2000);
	~L1_NEWTON();

	void newton(double *w);
	void set_print_string(void (*i_print) (const char *buf));
private:	
	double eps;
	double eta;
	int max_iter;
	int M;
	l1r_lr_fun *fun_obj;
	void info(const char *fmt,...);
	void pcg(double *pg, int max_cg, double *s, double *inv_diagH);
	void (*newton_print_string)(const char *buf);
};

class PROJGRAD
{
public:
	PROJGRAD(const l1r_lr_fun *fun_obj, double eps = 0.1, int m=10, double eta = 1e-4, int max_iter = 2000);
	~PROJGRAD();

	void projgrad(double *w);
	void set_print_string(void (*i_print) (const char *buf));
protected:
	double eps;
	double eta;
	int max_iter;
	int M;
	l1r_lr_fun *fun_obj;
	void info(const char *fmt,...);
	void (*projgrad_print_string)(const char *buf);
};

#define MEPS 1e-16L
#define PIVOTING
#define DEBUG_LDL

// solve Ax=b , in which A[i*n+j] = Aij and we only solve submatrix A[m:m]
double solve_system(const double *_A, const double *_b, int n, int m, double *x)
{
	long double *A = new long double[n * n];
	for(int i=0; i<n*n; i++)
		A[i] = _A[i];
	long double *b = new long double[n];
	for(int i=0; i<n; i++)
		b[i] = _b[i];

#ifdef PIVOTING
	int *p = new int[m];
	for(int i=0; i<m; i++)
		p[i] = i;
#endif

	// decomp
	// A -> D0
	//      L00 D1
	//      L10 L11 D2
	for(int j=0; j<m; j++){
#ifdef PIVOTING
		int pj = j;
		long double pj_val = fabsl(A[j*n+j]);
		// find piv
		for(int i=j+1; i<m; i++){
			long double pi_val = fabsl(A[i*n+i]);
			if(pj_val < pi_val){
				pj_val = pi_val;
				pj = i;
			}
		}
		// swap rows and columns of j and pj
		for(int i=0; i<m; i++)
			swap(A, j*n+i, pj*n+i);
		for(int i=0; i<m; i++)
			swap(A, i*n+j, i*n+pj);
		swap(b, j, pj);
		swap(p, j, pj);
#endif

		// loop invariant: Aik = Lik, Akk = Dk with k<j
		long double Dj = A[j*n+j];
		for(int k=0; k<j; k++){
			Dj -= A[j*n+k]*A[j*n+k] * A[k*n+k];
		}
		A[j*n+j] = Dj;
		//assert(fabs(Dj) >= MEPS);
		if(fabsl(Dj) < MEPS){
			for(int i=j+1; i<m; i++)
				A[i*n+j] = 0;
			continue; // xj should be 0
		}

		for(int i=j+1; i<m; i++){
			long double Lij = A[i*n+j];
			for(int k=0; k<j; k++){
				Lij -= A[i*n+k] * A[j*n+k] * A[k*n+k];
			}
			Lij /= Dj;
			A[i*n+j] = Lij;
		}
	}
#if 0
	printf("L\n");
	for(int i=0; i<n; i++){
		for(int j=0; j<=i; j++)
			printf("%+.3Le ", A[i*n+j]);
		printf("\n");
	}
#endif
	// invariant: A[i*n+j] = Lij, forall i>j
	//            A[i*n+i] = Dii, forall i

	// LD y = b
	// di yi =  bi - \sum_{j<i} yj Lij
	// place y in b
	for(int i=0; i<m; i++){
		// loop invariant: bk = yk forall k<i
		long double yi = b[i];
		for(int k=0; k<i; k++)
			yi -= A[i*n+k] * b[k];
		b[i] = yi;
	}
	for(int i=0; i<m; i++){
		long double Di = A[i*n+i];
		if(fabsl(Di) < MEPS)
			b[i] = 0;
		else
			b[i] /= Di;
	}
#if 0
	printf("y\n");
	for(int i=0; i<n; i++){
		printf("%+.3e\n", b[i]);
	}
#endif
	// L'x = y
	// xi = yi - \sum{j>i} xj Lij
	for(int i=m-1; i>=0; i--){
		// loop invariant: xk ready forall k>i
		long double xi = b[i];
#ifdef PIVOTING
		for(int k=m-1; k>i; k--)
			xi -= A[k*n+i] * x[p[k]];
		x[p[i]] = (double)xi;
#else
		for(int k=m-1; k>i; k--)
			xi -= A[k*n+i] * x[k];
		x[i] = (double)xi;
#endif
	}
#if 0
	printf("\n");
	for(int i=0; i<n; i++)
		printf("d %.3e\n", x[i]);

	printf("A\n");
	for(int i=0; i<n; i++){
		for(int j=0; j<n; j++)
			printf("%+.3e ", Acopy[i*n+j]);
		printf("\n");
	}
	printf("b\n");
	for(int i=0; i<n; i++){
		printf("%+.3e\n", bcopy[i]);
	}
#endif

#if 1
	double err = 0;
	for(int i=0; i<m; i++){
		double Ax_i = 0;
		for(int j=0; j<m; j++)
			Ax_i += _A[i*n+j] * x[j];
		err += fabs(Ax_i-_b[i]);
	}
	double max_pv = 0;
	for(int i=0; i<m; i++)
		if(max_pv < _A[i*n+i])
			max_pv = (double)_A[i*n+i];
	if(max_pv != 0)
		err /= max_pv;
	//printf("err=%e\n", err);
	//assert(err < 1e-5);
#endif
	delete[] A;
	delete[] b;
#ifdef PIVOTING
	delete[] p;
#endif
	return err;
}

#ifdef __cplusplus
extern "C" {
#endif

extern double dnrm2_(int *, double *, int *);
extern double ddot_(int *, double *, int *, double *, int *);
extern int daxpy_(int *, double *, double *, int *, double *, int *);
extern int dscal_(int *, double *, double *, int *);
extern int dcopy_(int *, double *, int *, double *, int *);	
extern int dgemv_(char *, int *, int *, double *, double *, int *, double *, int *, double *, double *, int *);
extern int dsyrk_(char *, char *, int *, int *, double *, double *, int *, double *, double *, int *);

#ifdef __cplusplus
}
#endif

static void default_print(const char *buf)
{
	fputs(buf,stdout);
	fflush(stdout);
}

void OWLQN::info(const char *fmt,...)
{
	if(mpi_get_rank()!=0)
		return;
	char buf[BUFSIZ];
	va_list ap;
	va_start(ap,fmt);
	vsprintf(buf,fmt,ap);
	va_end(ap);
	(*owlqn_print_string)(buf);
}

void OWLQN::set_print_string(void (*print_string) (const char *buf))
{
	owlqn_print_string = print_string;
}

OWLQN::OWLQN(const l1r_lr_fun *fun_obj, double eps, int m, double eta, int max_iter)
{
	this->fun_obj=const_cast<l1r_lr_fun *>(fun_obj);
	this->eps=eps;
	this->max_iter=max_iter;
	this->eta = eta;
	owlqn_print_string = default_print;
	this->M = (m-1)/2;
	int w_size = this->fun_obj->get_nr_variable();
	int nr_node = mpi_get_size();
	int rank = mpi_get_rank();
	int shift = int(ceil(double(w_size) / double(nr_node)));
	this->recv_count = new int[nr_node];
	this->displace = new int[nr_node];
	this->start = shift * rank;
	this->length = min(max(w_size - start, 0), shift);
	if (length == 0)
		start = 0;
	int counter = 0;
	for (int i=0;i<nr_node;i++)
	{
		recv_count[i] = shift;
		displace[i] = counter;
		counter += shift;
		if (counter >= w_size)
		{
			counter = 0;
			shift = 0;
		}
		else if (counter + shift > w_size)
			shift = w_size - counter;
	}
}

OWLQN::~OWLQN()
{
	delete[] recv_count;
	delete[] displace;
}

void OWLQN::owlqn(double *w, int solver_type_)
{
	enum{TWOLOOP,VF};
	int solver_type = 0;
	switch (solver_type_)
	{
		case L1R_LR_OWLQN:
		case L1R_LR_OWLQN_COL:
		{
			solver_type = TWOLOOP;
			break;
		}
		case L1R_LR_OWLQN_VF:
		case L1R_LR_OWLQN_VF_COL:
		{
			solver_type = VF;
			break;			
		}
	}

	int n = fun_obj->get_nr_variable();
	int i;
	int k = 0;
	double f;
	int iter = 1, inc = 1;
	double *loss_g = new double[n];
	double *pg = new double[n];
	double *old_w = new double[n];
	double *old_loss_g = new double[length];
	double *step = new double[n];
	double *local_step = new double[length];
	double pgnorm;
	int nnz_ops = 0;	
	double init_step_size = 1;
	double step_size;
	int num_linesearch;
	int64_t timer_st = wall_clock_ns(), timer_ed;
	char N[] = "N";
	double one = 1.0;
	double **inner_product_matrix = new double*[2*M+1];
	double* delta = new double[M * 2 + 1];
	for (i=0; i < 2 * M + 1; i++)
		inner_product_matrix[i] = new double[2*M+1];

	// calculate gradient norm at w=0 for stopping condition.
	double *w0 = new double[n];
	for (i=0; i<n; i++)
		w0[i] = 0;
	fun_obj->fun(w0);
	nnz_ops++;	
	fun_obj->loss_grad(w0, loss_g);
	nnz_ops++;	
	fun_obj->cal_pg(w0, loss_g, pg);
	double pgnorm0 = dnrm2_(&n, pg, &inc);
	delete [] w0;

	iter = 0;

	double *s = new double[M*length];
	double *y = new double[M*length];
	double *rho = new double[M];

	f = fun_obj->fun(w);
	nnz_ops++;	

	while (iter < max_iter)
	{
		int DynamicM = min(iter, M);

		fun_obj->loss_grad(w, loss_g);
		nnz_ops++;		
		fun_obj->cal_pg(w, loss_g, pg);
		int nnz = 0;
		int w0_gsmall = 0, w0_glarge = 0;
		for (i = 0; i < n; i++)
		{
			old_w[i] = w[i];
			if (w[i] != 0)
				nnz++;
			else
			{
				if (fabs(loss_g[i]) > 1)
					w0_glarge++;
				else
					w0_gsmall++;
			}
		}

		double s0y0 = 0;
		if (iter == 0)
		{
			pgnorm = ddot_(&length, pg + start, &inc, pg + start, &inc);
			mpi_allreduce(&pgnorm, 1, MPI_DOUBLE, MPI_SUM);
			pgnorm = sqrt(pgnorm);
			if (solver_type != TWOLOOP)
				inner_product_matrix[0][0] = pgnorm * pgnorm;
		}
		else
		{
			for (i=0;i<length;i++)
				y[k*length+i] = loss_g[start + i] - old_loss_g[i];
			if (solver_type == TWOLOOP)
			{
				pgnorm = ddot_(&length, pg + start, &inc, pg + start, &inc);
				mpi_allreduce(&pgnorm, 1, MPI_DOUBLE, MPI_SUM);
				pgnorm = sqrt(pgnorm);
				for (i=0;i<length;i++)
					s0y0 += y[k*length + i] * s[k*length + i];
				mpi_allreduce(&s0y0, 1, MPI_DOUBLE, MPI_SUM);
			}
			else
			{
				update_inner_products(inner_product_matrix, k, DynamicM, s, y, pg);
				s0y0 = inner_product_matrix[2 * k][2* k + 1];
				pgnorm = sqrt(inner_product_matrix[2 * DynamicM][2 * DynamicM]);
			}
		}

		if (pgnorm <= eps*pgnorm0)
		{
			info("final pgnorm %g\n", pgnorm);
			break;
		}

		timer_ed = wall_clock_ns();
		info("iter=%d m=%d |pg|=%5.3e f=%.14e nnz_ops=%d nnz=%d elapsed_time=%g w0_gsmall=%d w0_glarge=%d\n", iter, 2 * DynamicM + 1, pgnorm, f, nnz_ops, nnz, wall_time_diff(timer_ed, timer_st), w0_gsmall, w0_glarge);

		if (iter > 0)
		{
			if (s0y0 == 0)
				break;
			rho[k] = 1.0 / s0y0;
			k = (k+1)%M;

			if (solver_type == TWOLOOP)
				TwoLoop(pg, s, y, rho, DynamicM, local_step, k, length);
			else
			{
				TwoLoop(inner_product_matrix, rho, DynamicM, k, delta);
				memset(local_step, 0, sizeof(double) * length);
				daxpy_(&length, delta + 2 * DynamicM, pg + start, &inc, local_step, &inc);
				double *tmpdelta = new double[DynamicM];
				for (i=0;i<DynamicM;i++)
					tmpdelta[i] = delta[2 * i];
				dgemv_(N, &length, &DynamicM, &one, s, &length, tmpdelta, &inc, &one, local_step, &inc);
				for (i=0;i<DynamicM;i++)
					tmpdelta[i] = delta[2 * i + 1];
				dgemv_(N, &length, &DynamicM, &one, y, &length, tmpdelta, &inc, &one, local_step, &inc);
				delete[] tmpdelta;
			}
			for (i=0;i<length; i++)
				local_step[i] *= (local_step[i] * pg[start + i] < 0);
			MPI_Allgatherv(local_step, length, MPI_DOUBLE, step, recv_count, displace, MPI_DOUBLE, MPI_COMM_WORLD);
			
			init_step_size = 1;
		}
		else
		{
			for (i=0;i<n;i++)
				step[i] = -pg[i];
			init_step_size = 1/pgnorm;
		}

		if (iter == 0 || iter == 1)
			step_size = init_step_size;
		else if (iter %4 == 0)
			step_size = min(init_step_size, step_size*2);

		f = fun_obj->line_search(step, old_w, loss_g, pg, &step_size, eta, f, &num_linesearch, w);
		nnz_ops += num_linesearch;

		nnz = 0;
		for (i=0; i<length; i++)
		{
			old_loss_g[i] = loss_g[start + i];
			s[k * length + i] = w[start + i] - old_w[start + i];
			if (w[i] != 0)
				nnz++;
		}

		iter++;
	}
	delete[] s;
	delete[] y;
	delete[] loss_g;
	delete[] old_loss_g;
	delete[] pg;
	delete[] step;
	delete[] old_w;
	delete[] local_step;
	for (i=0; i < 2 * M + 1; i++)
		delete[] inner_product_matrix[i];
	delete[] inner_product_matrix;
	delete[] delta;
}

void OWLQN::TwoLoop(double* g, double* s, double* y, double* rho, int DynamicM, double *step, int k, int n)
{
	int i, j;
	int inc = 1;
	double *alpha = new double[DynamicM];
	double tmp;
	for (i=0;i<n;i++)
		step[i] = -g[this->start + i];
	int start = k-1;
	if (k < DynamicM)
		start += DynamicM;

	int lastrho = -1;
	for (i = 0; i < DynamicM; i++)
	{
		j = start % DynamicM;

		start--;
		if (rho[j] > 0)
		{
			tmp = ddot_(&n, step, &inc, s + j*n, &inc);
			mpi_allreduce(&tmp, 1, MPI_DOUBLE, MPI_SUM);
			alpha[j] = rho[j] * tmp;
			double a = -alpha[j];
			daxpy_(&n, &a, y + j*n, &inc, step, &inc);
			if (lastrho == -1)
				lastrho = j;
		}
		else
		{
			fprintf(stderr,"ERROR: rho[%d] <= 0\n",i);
			return;
		}
	}
	if (lastrho != -1)
	{
		tmp = ddot_(&n, y + lastrho*n, &inc, y + lastrho*n, &inc);
		mpi_allreduce(&tmp, 1, MPI_DOUBLE, MPI_SUM);
		double hk0 = 1.0/(rho[lastrho] * tmp);
		dscal_(&n, &hk0, step, &inc);
		for (i = 0; i < DynamicM; i++)
		{
			start++;
			j = start % DynamicM;
			if (rho[j] <= 0)
				continue;
			tmp = ddot_(&n, step, &inc, y + j*n, &inc);
			mpi_allreduce(&tmp, 1, MPI_DOUBLE, MPI_SUM);
			double beta = rho[j] * tmp;
			beta = alpha[j] - beta;
			daxpy_(&n, &beta, s + j*n, &inc, step, &inc);
		}
	}
}

void OWLQN::update_inner_products(double** inner_product_matrix, int k, int DynamicM, double* s, double* y, double* pg)
{
	int i;
	int inc = 1;
	double* buffer = new double[DynamicM * 6 + 1];
	memset(buffer, 0, sizeof(double) * DynamicM * 6 + 1);
	for (i=0;i<DynamicM;i++)
	{
		buffer[6 * i] = ddot_(&length, s + k * length, &inc,  s + i * length, &inc);
		buffer[6 * i + 1] = ddot_(&length, s + k * length, &inc,  y + i * length, &inc);
		buffer[6 * i + 2] = ddot_(&length, y + k * length, &inc,  s + i * length, &inc);
		buffer[6 * i + 3] = ddot_(&length, y + k * length, &inc,  y + i * length, &inc);
		buffer[6 * i + 4] = ddot_(&length, pg + start, &inc,  s + i * length, &inc);
		buffer[6 * i + 5] = ddot_(&length, pg + start, &inc,  y + i * length, &inc);
	}
	buffer[6 * DynamicM] = ddot_(&length, pg + start, &inc,  pg + start, &inc);

	mpi_allreduce(buffer, 6 * DynamicM + 1, MPI_DOUBLE, MPI_SUM);
	for (i=0;i<DynamicM;i++)
	{
		inner_product_matrix[2 * i][2 * k] = buffer[6 * i];
		inner_product_matrix[2 * i + 1][2 * k] = buffer[6 * i + 1];
		inner_product_matrix[2 * i][2 * k + 1] = buffer[6 * i + 2];
		inner_product_matrix[2 * i + 1][2 * k + 1] = buffer[6 * i + 3];
		inner_product_matrix[2 * i][2 * DynamicM] = buffer[6 * i + 4];
		inner_product_matrix[2 * i + 1][2 * DynamicM] = buffer[6 * i + 5];

		inner_product_matrix[2 * k][2 * i] = inner_product_matrix[2 * i][2 * k];
		inner_product_matrix[2 * k][2 * i + 1] = inner_product_matrix[2 * i + 1][2 * k];
		inner_product_matrix[2 * k + 1][2 * i] = inner_product_matrix[2 * i][2 * k + 1];
		inner_product_matrix[2 * k + 1][2 * i + 1] = inner_product_matrix[2 * i + 1][2 * k + 1];
		inner_product_matrix[2 * DynamicM][2 * i] = inner_product_matrix[2 * i][2 * DynamicM];
		inner_product_matrix[2 * DynamicM][2 * i + 1] = inner_product_matrix[2 * i + 1][2 * DynamicM];
	}
	inner_product_matrix[2 * DynamicM][2 * DynamicM] = buffer[6 * DynamicM];

	delete[] buffer;
}

void OWLQN::TwoLoop(double** inner_product_matrix, double* rho, int DynamicM, int k, double *delta)
{
	int i, j;
	int inc = 1;
	int start = k-1;
	double beta;
	double *alpha = new double[DynamicM];
	int tmp_length = 2 * DynamicM + 1;

	memset(delta, 0, sizeof(double) * size_t(2 * DynamicM));
	memset(alpha, 0, sizeof(double) * size_t(DynamicM));
	delta[2 * DynamicM] = -1;
	if (k < DynamicM)
		start += DynamicM;

	int lastrho = -1;
	for (i = 0; i < DynamicM; i++)
	{
		j = start % DynamicM;

		start--;
		if (rho[j] > 0)
		{
			alpha[j] = rho[j] * ddot_(&tmp_length, delta, &inc, inner_product_matrix[2 * j], &inc);
			delta[2 * j + 1] -= alpha[j];
			if (lastrho == -1)
				lastrho = j;
		}
		else
		{
			fprintf(stderr,"ERROR: rho[%d] <= 0\n",i);
			return;
		}
	}
	if (lastrho != -1)
	{
		double scal = (1.0 / (rho[lastrho] * inner_product_matrix[2 * lastrho + 1][2 * lastrho + 1]));
		dscal_(&tmp_length, &scal, delta, &inc);
		for (i = 0; i < DynamicM; i++)
		{
			start++;
			j = start % DynamicM;
			if (rho[j] <= 0)
				continue;
			beta = alpha[j] - rho[j] * ddot_(&tmp_length, delta, &inc, inner_product_matrix[2 * j + 1], &inc);
			delta[2 * j] += beta;
		}
	}
	delete[] alpha;
}


L1_NEWTON::L1_NEWTON(const l1r_lr_fun *fun_obj, double eps, int m, double eta, int max_iter)
{
	this->fun_obj=const_cast<l1r_lr_fun *>(fun_obj);
	this->eps=eps;
	this->max_iter=max_iter;
	this->eta = eta;
	newton_print_string = default_print;
	this->M = m;
}


L1_NEWTON::~L1_NEWTON()
{
}

void L1_NEWTON::newton(double *w)
{
	int n = fun_obj->get_nr_variable();
	//	int l = fun_obj->get_nr_data();
	int i;
	double f;
	int iter = 1, inc = 1;
	double *loss_g = new double[n];
	double *pg = new double[n];
	double *old_w = new double[n];
	double *step = new double[n];
	double pgnorm;
	int nnz_ops = 0;
	double init_step_size = 1;
	double step_size;
	int num_linesearch;
	double *inv_diagH = new double[n];
	int DynamicM = 0;	
	int64_t timer_st = wall_clock_ns(), timer_ed;
		
	// calculate gradient norm at w=0 for stopping condition.
	double *w0 = new double[n];
	for (i=0; i<n; i++)
		w0[i] = 0;
	fun_obj->fun(w0);
	nnz_ops++;	
	fun_obj->loss_grad(w0, loss_g);
	nnz_ops++;	
	fun_obj->cal_pg(w0, loss_g, pg);
	double pgnorm0 = dnrm2_(&n, pg, &inc);
	delete [] w0;

	iter = 0;

	f = fun_obj->fun(w);
	nnz_ops++;	

	while (iter < max_iter)
	{
		DynamicM = min(iter+1, M);
		
		fun_obj->loss_grad(w, loss_g);
		nnz_ops++;			
		fun_obj->cal_pg(w, loss_g, pg);

		pgnorm = dnrm2_(&n, pg, &inc);

		if (pgnorm <= eps*pgnorm0)
		{
			info("final pgnorm %g\n", pgnorm);
			break;
		}

		fun_obj->get_diag(inv_diagH);
		for (i=0; i<n; i++)
		{
			inv_diagH[i] = 0.01*inv_diagH[i]+0.99;
			inv_diagH[i] = 1/inv_diagH[i];
		}
		pcg(pg, DynamicM, step, inv_diagH);		
		nnz_ops += 2*DynamicM;
		
		for (i=0;i<n; i++)
			if (step[i]*pg[i] >= 0)
				step[i] = 0;

		int nnz = 0;
		int w0_gsmall = 0, w0_glarge = 0;
		for (i = 0; i < n; i++)
		{
			old_w[i] = w[i];
			if (w[i] != 0)
				nnz++;
			else
			{
				if (fabs(loss_g[i]) > 1)
					w0_glarge++;
				else
					w0_gsmall++;
			}
		}

		timer_ed = wall_clock_ns();
		info("iter=%d m=%d |pg|=%5.3e f=%.14e nnz_ops=%d nnz=%d elapsed_time=%g w0_gsmall=%d w0_glarge=%d\n", iter, DynamicM, pgnorm, f, nnz_ops, nnz, wall_time_diff(timer_ed, timer_st), w0_gsmall, w0_glarge);

		if (iter == 0)
			step_size = init_step_size;
		else if (iter %4 == 0)
			step_size = min(init_step_size, step_size*2);

		f = fun_obj->line_search(step, old_w, loss_g, pg, &step_size, eta, f, &num_linesearch, w);
		nnz_ops += num_linesearch;		
		
		iter++;
	}

	delete[] inv_diagH;
	delete[] loss_g;
	delete[] old_w;
	delete[] step;
}

void L1_NEWTON::pcg(double *pg, int max_cg, double *s, double *inv_diagH)
{
	int i, inc = 1;
	int n = fun_obj->get_nr_variable();
	double one = 1;
	double *d = new double[n];
	double *Hd = new double[n];
	double *r = new double[n];
	double *z = new double[n];	
	double zTr, zTr_new, alpha, beta;
	int iter=0;
	
	for (i=0; i<n; i++)
	{
		s[i] = 0;
		r[i] = -pg[i];
		z[i] = inv_diagH[i] * r[i];
		d[i] = z[i];
	}
	
	zTr = ddot_(&n, z, &inc, r, &inc);

	for (iter=0; iter<max_cg; iter++)
	{
		fun_obj->Hv(d, Hd);

		for (i=0; i<n; i++)
			if (pg[i] == 0)
				Hd[i] = 0;
		
		alpha = zTr/ddot_(&n, d, &inc, Hd, &inc);
		daxpy_(&n, &alpha, d, &inc, s, &inc);
	
		alpha = -alpha;
		daxpy_(&n, &alpha, Hd, &inc, r, &inc);

		for (i=0; i<n; i++)
			z[i] = inv_diagH[i] * r[i];
		zTr_new = ddot_(&n, z, &inc, r, &inc);
		beta = zTr_new/zTr;
		dscal_(&n, &beta, d, &inc);
		daxpy_(&n, &one, z, &inc, d, &inc);
		zTr = zTr_new;
	}
	
	delete[] d;
	delete[] Hd;
	delete[] r;
	delete[] z;
}

void L1_NEWTON::info(const char *fmt,...)
{
	if(mpi_get_rank()!=0)
		return;
	char buf[BUFSIZ];
	va_list ap;
	va_start(ap,fmt);
	vsprintf(buf,fmt,ap);
	va_end(ap);
	(*newton_print_string)(buf);
}

void L1_NEWTON::set_print_string(void (*print_string) (const char *buf))
{
	newton_print_string = print_string;
}

L1_LCOMM::L1_LCOMM(const l1r_lr_fun *fun_obj, double eps, int m, double eta, int max_iter)
{
	this->fun_obj=const_cast<l1r_lr_fun *>(fun_obj);
	this->eps=eps;
	this->max_iter=max_iter;
	this->eta = eta;
	lcomm_print_string = default_print;
	this->M = m;
	int w_size = this->fun_obj->get_nr_variable();
	int nr_node = mpi_get_size();
	int rank = mpi_get_rank();
	int shift = int(ceil(double(w_size) / double(nr_node)));
	this->recv_count = new int[nr_node];
	this->displace = new int[nr_node];
	this->start = shift * rank;
	this->length = min(max(w_size - start, 0), shift);
	if (length == 0)
		start = 0;
	int counter = 0;
	for (int i=0;i<nr_node;i++)
	{
		recv_count[i] = shift;
		displace[i] = counter;
		counter += shift;
		if (counter >= w_size)
		{
			counter = 0;
			shift = 0;
		}
		else if (counter + shift > w_size)
			shift = w_size - counter;
	}
}


L1_LCOMM::~L1_LCOMM()
{
	delete[] recv_count;
	delete[] displace;
}

void L1_LCOMM::lcomm(double *w, int solver_type_)
{

	enum { PG, STEP, LBFGS };
	int solver_type;
	switch (solver_type_)
	{
		case L1R_LR_LCOMM_PG:
		case L1R_LR_LCOMM_PG_COL:
		{
			solver_type = PG;
			break;
		}
		case L1R_LR_LCOMM_STEP:
		case L1R_LR_LCOMM_STEP_COL:
		{
			solver_type = STEP;
			break;
		}
		case L1R_LR_LCOMM_LBFGS:
		case L1R_LR_LCOMM_LBFGS_COL:
		{
			solver_type = LBFGS;
			break;
		}
	}

	int n = fun_obj->get_nr_variable();
	int l = fun_obj->get_nr_data();
	int i;
	int k = 0;
	double f;
	int iter = 1;
	double *loss_g = new double[n];
	double *pg = new double[n];
	double *old_w = new double[n];
	double *localstep = new double[length];
	double *step = new double[n];
	double pgnorm;
	double *t = new double[M];
	double *XP = new double[M*l];
	double *P = new double[M*length];
	double* rhs = new double[M];
	double *sqrtCD = new double[l];
	double *Xw;
	double *old_Xw = new double[l];
	double *Xpg = new double[l];
	double *system = new double[M * M];
	double err = 0;
	int DynamicM = 0;
	int nnz_ops = 0;	
	double init_step_size = 1;
	double step_size;
	int num_linesearch;
	int64_t timer_st = wall_clock_ns(), timer_ed;

	// for blas
	double one = 1.0;
	double mone = -1.0;	
	double zero = 0.0;
	int inc = 1;
	char N[] = "N";
	char T[] = "T";
	
	// calculate gradient norm at w=0 for stopping condition.
	double *w0 = new double[n];
	for (i=0; i<n; i++)
		w0[i] = 0;
	fun_obj->fun(w0);
	nnz_ops++;	
	fun_obj->loss_grad(w0, loss_g);
	nnz_ops++;	
	fun_obj->cal_pg(w0, loss_g, pg);
	double pgnorm0 = dnrm2_(&n, pg, &inc);
	delete [] w0;

	iter = 0;

	f = fun_obj->fun(w);
	Xw = fun_obj->get_Xw();
	nnz_ops++;	

	while (iter < max_iter)
	{
		Xpg = XP + k*l;
		fun_obj->loss_grad(w, loss_g);
		nnz_ops++;		
		fun_obj->cal_pg(w, loss_g, pg);
		fun_obj->Xv(pg, Xpg);
		nnz_ops++;
		memcpy(P + k * length, pg + start, sizeof(double) * length);
		k = (k+1)%M;
		DynamicM++;

		pgnorm = dnrm2_(&n, pg, &inc);

		if (pgnorm <= eps*pgnorm0)
		{
			info("final pgnorm %g\n", pgnorm);
			break;
		}

		fun_obj->cal_sqrtCD(sqrtCD);
		DynamicM = min(DynamicM, M);
		
		compute_system(iter, DynamicM, l, XP, sqrtCD, system);
		
		dgemv_(T, &length, &DynamicM, &mone, P, &length, pg + start, &inc, &zero, rhs, &inc);		
		mpi_allreduce(rhs, DynamicM, MPI_DOUBLE, MPI_SUM);
		//parallelize all rhs computation by one O(m) communication
		
		err = solve_system(system, rhs, DynamicM, DynamicM, t);
		
		dgemv_(N, &length, &DynamicM, &one, P, &length, t, &inc, &zero, localstep, &inc);

		for (i=0;i<length; i++)
			if (localstep[i]*pg[start + i] >= 0)
				localstep[i] = 0;
		MPI_Allgatherv(localstep, length, MPI_DOUBLE, step, recv_count, displace, MPI_DOUBLE, MPI_COMM_WORLD);

		int nnz = 0;
		int w0_gsmall = 0, w0_glarge = 0;
		for (i = 0; i < n; i++)
		{
			old_w[i] = w[i];
			if (w[i] != 0)
				nnz++;
			else
			{
				if (fabs(loss_g[i]) > 1)
					w0_glarge++;
				else
					w0_gsmall++;
			}
		}

		timer_ed = wall_clock_ns();
		info("iter=%d m=%d |pg|=%5.3e f=%.14e nnz_ops=%d nnz=%d elapsed_time=%g\n", iter, DynamicM, pgnorm, f, nnz_ops, nnz, wall_time_diff(timer_ed, timer_st));
		
		if (solver_type != PG)
			dcopy_(&l, Xw, &inc, old_Xw, &inc);

		if (iter == 0)
			step_size = init_step_size;
		else if (iter %4 == 0)
			step_size = min(init_step_size, step_size*2);

		f = fun_obj->line_search(step, old_w, loss_g, pg, &step_size, eta, f, &num_linesearch, w);
		nnz_ops += num_linesearch;

		if (solver_type == STEP)
		{
			if (k == 0 && DynamicM == M)
				k = M-1;
			else
				k = max(k-1, 0);
			DynamicM--;
		}
		
		if (solver_type != PG)
		{
			Xw = fun_obj->get_Xw();					
			for (i=0; i<length; i++)
				P[k * length + i] = w[start + i] - old_w[start + i];
			for (i=0; i<l; i++)
				XP[k*l + i] = Xw[i] - old_Xw[i];
			k = (k+1)%M;
			DynamicM++;
		}
		
		iter++;
	}

	delete[] XP;
	delete[] P;
	delete[] loss_g;
	delete[] old_w;
	delete[] system;
	delete[] t;
	delete[] rhs;
	delete[] sqrtCD;
	delete[] old_Xw;
	delete[] step;
}

/*
void L1_LCOMM::lcomm_LBFGS(double *w)
{
	int n = fun_obj->get_nr_variable();
	int l = fun_obj->get_nr_data();
	int i;
	int k = 0;
	double f;
	int iter = 1, inc = 1;
	double *loss_g = new double[n];
	double *pg = new double[n];
	double *old_pg = new double[n];
	double *old_w = new double[n];
	double *step;
	double *step_x;
	double pgnorm;
	double *delta = new double[M];
	double **XP = new double*[M];
	double **p = new double*[M];
	double* rhs = new double[M];
	double *sqrtCD = new double[l];
	double *system = new double[M * M];
	double err = 0;
	int DynamicM = 0;
	int total_fun_eval = 0;
	double init_step_size = 1;
	double step_size;
	int num_linesearch;
	// double diag_factor = 0.01;

	// calculate gradient norm at w=0 for stopping condition.
	double *w0 = new double[n];
	for (i=0; i<n; i++)
		w0[i] = 0;
	fun_obj->fun(w0);
	fun_obj->loss_grad(w0, loss_g);
	fun_obj->cal_pg(w0, loss_g, pg);
	double pgnorm0 = dnrm2_(&n, pg, &inc);
	delete [] w0;

	iter = 0;

	for (int i = 0; i < M; i++)
		p[i] = NULL;

	f = fun_obj->fun(w);
	total_fun_eval++;

	while (iter < max_iter)
	{
		fun_obj->loss_grad(w, loss_g);
		fun_obj->cal_pg(w, loss_g, pg);

		pgnorm = dnrm2_(&n, pg, &inc);

		if (pgnorm <= eps*pgnorm0)
		{
			info("final pgnorm %g\n", pgnorm);
			p[k] = NULL; // otherwise it tries to free old_pg
			delete[] XP[k];
			break;
		}

		if (iter == 0)
		{
			if (p[k] != NULL)
			{
				delete[] p[k];
				delete[] XP[k];
			}
			XP[k] = new double[l];
			p[k] = pg;
			info("X * pg ");
			fun_obj->Xv(p[k], XP[k]);
			DynamicM++;
		}

		DynamicM = min(DynamicM, M);

		int nnz = 0;
		for (i = 0; i < n; i++)
		{
			old_w[i] = w[i];
			if (w[i] != 0)
				nnz++;
		}

		info("iter=%d m=%d |pg|=%5.3e f=%.14e total_fun_eval=%d nnz=%d\n", iter, DynamicM, pgnorm, f, total_fun_eval, nnz);

		fun_obj->cal_sqrtCD(sqrtCD);
		compute_system(iter, DynamicM, l, XP, sqrtCD, system);

		for (i=0;i<DynamicM;i++)
			rhs[i] = -ddot_(&n, pg, &inc, p[i], &inc);

//		for (i=0;i<DynamicM;i++)
//			system[i*DynamicM + i] += diag_factor;
//

		err = solve_system(system, rhs, DynamicM, DynamicM, delta);
		step = new double[n];
		memset(step, 0, sizeof(double) * size_t(n));

		for (i=0;i<DynamicM;i++)
			daxpy_(&n, delta+i, p[i], &inc, step, &inc);

		for (i=0;i<n; i++)
			if (step[i]*pg[i] >= 0)
				step[i] = 0;

		double old_f = f;

		if (iter == 0)
			step_size = init_step_size;
		else if (iter %4 == 0)
			step_size = min(init_step_size, step_size*2);

		f = fun_obj->line_search(step, old_w, loss_g, pg, &step_size, eta, f, &num_linesearch, w);
		total_fun_eval += num_linesearch;

//		double prered = step_size*(1-step_size/2)*ddot_(&DynamicM, rhs, &inc, delta, &inc);
//		double actred = old_f - f;
//		info("actred %g prered %g\n", actred, prered);
//		if (actred < 0.25*prered)
//			diag_factor = diag_factor * 3.0 / 2.0;
//		else if (actred > 0.75*prered)
//			diag_factor = diag_factor * 2.0 / 3.0;
//		info("diag_factor %g\n", diag_factor);

		for (i=0; i<n; i++)
		{
			step[i] = w[i] - old_w[i];
			//			old_pg[i] = pg[i];
		}
		delete[] XP[k];
		p[k] = NULL;
		DynamicM--;

		// comment this part if we don't want to use step difference
		// but need to delete step
		p[k] = step;
		step_x = new double[l];
		info("X * step ");
		fun_obj->Xv(step, step_x);
		XP[k] = step_x;
		k = (k+1)%M;
		DynamicM++;

		if (iter > 0)
		{
			if (p[k] != NULL)
			{
				delete[] p[k];
				delete[] XP[k];
			}
			p[k] = new double[n];
			XP[k] = new double[l];
			for (int i = 0; i < n; i++)
				p[k][i] = pg[i]-old_pg[i];
			info("X * pg_diff ");
			fun_obj->Xv(p[k], XP[k]);
			k = (k+1)%M;
			DynamicM++;

			for (i=0; i<n; i++)
				old_pg[i] = pg[i];

		}

		if (p[k] != NULL)
		{
			delete[] p[k];
			delete[] XP[k];
		}
		XP[k] = new double[l];
		p[k] = old_pg;
		info("X * pg ");
		fun_obj->Xv(p[k], XP[k]);
		DynamicM++;

		iter++;
	}

	for (i=0;i<min(DynamicM,M);i++)
	{
		if (p[i] != NULL)
		{
			delete[] XP[i];
			delete[] p[i];
		}
	}
	delete[] XP;
	delete[] p;
	delete[] loss_g;
	delete[] pg;
	delete[] old_pg;
	delete[] old_w;
	delete[] system;
	delete[] delta;
	delete[] rhs;
	delete[] sqrtCD;
}
*/

void L1_LCOMM::compute_system(int iter, int DynamicM, int l, double* XP, double* sqrtCD, double* system)
{
	int i, j;

	double *sqrtCD_XP = new double[DynamicM*l];
	char U[] = "U";
	char T[] = "T";
	double one = 1.0;
	double zero = 0.0;

	for (i=0;i<DynamicM;i++)
		for (j=0;j<l;j++)
			sqrtCD_XP[i*l + j] = sqrtCD[j] * XP[i*l + j];

	dsyrk_(U, T, &DynamicM, &l, &one, sqrtCD_XP, &l, &zero, system, &DynamicM);

	mpi_allreduce(system, DynamicM * DynamicM, MPI_DOUBLE, MPI_SUM);
	
	for (j=0;j<DynamicM;j++)
		for (i=j+1;i<DynamicM;i++)
			system[j * DynamicM + i] = system[i * DynamicM + j];

	delete[] sqrtCD_XP;
}

void L1_LCOMM::info(const char *fmt,...)
{
	if(mpi_get_rank()!=0)
		return;
	char buf[BUFSIZ];
	va_list ap;
	va_start(ap,fmt);
	vsprintf(buf,fmt,ap);
	va_end(ap);
	(*lcomm_print_string)(buf);
}

void L1_LCOMM::set_print_string(void (*print_string) (const char *buf))
{
	lcomm_print_string = print_string;
}


PROJGRAD::PROJGRAD(const l1r_lr_fun *fun_obj, double eps, int m, double eta, int max_iter)
{
	this->fun_obj=const_cast<l1r_lr_fun *>(fun_obj);
	this->eps=eps;
	this->max_iter=max_iter;
	this->eta = eta;
	projgrad_print_string = default_print;
}

PROJGRAD::~PROJGRAD()
{
}

void PROJGRAD::projgrad(double *w)
{
	int n = fun_obj->get_nr_variable();
	int i;
	double f;
	int iter = 1, inc = 1;
	double *loss_g = new double[n];
	double *pg = new double[n];
	double *old_w = new double[n];
	double *step = new double[n];
	double pgnorm;
	int nnz_ops = 0;
	double init_step_size = 1;
	double step_size;
	int num_linesearch;
	double *M = new double[n];
	int64_t timer_st = wall_clock_ns(), timer_ed;

	// calculate gradient norm at w=0 for stopping condition.
	double *w0 = new double[n];
	for (i=0; i<n; i++)
		w0[i] = 0;
	fun_obj->fun(w0);
	nnz_ops++;	
	fun_obj->loss_grad(w0, loss_g);
	nnz_ops++;	
	fun_obj->cal_pg(w0, loss_g, pg);
	double pgnorm0 = dnrm2_(&n, pg, &inc);
	delete [] w0;

	iter = 0;

	f = fun_obj->fun(w);
	nnz_ops++;	

	while (iter < max_iter)
	{
		fun_obj->loss_grad(w, loss_g);
		nnz_ops++;		
		fun_obj->cal_pg(w, loss_g, pg);

		pgnorm = dnrm2_(&n, pg, &inc);

		if (pgnorm <= eps*pgnorm0)
			break;

		int nnz = 0;
		int w0_gsmall = 0, w0_glarge = 0;
		for (i = 0; i < n; i++)
		{
			old_w[i] = w[i];
			if (w[i] != 0)
				nnz++;
			else
			{
				if (fabs(loss_g[i]) > 1)
					w0_glarge++;
				else
					w0_gsmall++;
			}
		}

		timer_ed = wall_clock_ns();
		info("iter=%d |pg|=%5.3e f=%.14e nnz_ops=%d nnz=%d elapsed_time=%g\n", iter, pgnorm, f, nnz_ops, nnz, wall_time_diff(timer_ed, timer_st));

		fun_obj->get_diag(M);

		for (i=0;i<n; i++)
			step[i] = -pg[i]/M[i];

		step_size = init_step_size;
		f = fun_obj->line_search(step, old_w, loss_g, pg, &step_size, eta, f, &num_linesearch, w);
		nnz_ops += num_linesearch;

		for (i=0; i<n; i++)
			step[i] = w[i] - old_w[i];

		iter++;
	}

	delete[] loss_g;
	delete[] pg;
	delete[] step;
	delete[] old_w;
	delete[] M;
}

void PROJGRAD::info(const char *fmt,...)
{
	char buf[BUFSIZ];
	va_list ap;
	va_start(ap,fmt);
	vsprintf(buf,fmt,ap);
	va_end(ap);
	(*projgrad_print_string)(buf);
}

void PROJGRAD::set_print_string(void (*print_string) (const char *buf))
{
	projgrad_print_string = print_string;
}

// end of l1 solvers


// A coordinate descent algorithm for
// multi-class support vector machines by Crammer and Singer
//
//  min_{\alpha}  0.5 \sum_m ||w_m(\alpha)||^2 + \sum_i \sum_m e^m_i alpha^m_i
//    s.t.     \alpha^m_i <= C^m_i \forall m,i , \sum_m \alpha^m_i=0 \forall i
//
//  where e^m_i = 0 if y_i  = m,
//        e^m_i = 1 if y_i != m,
//  C^m_i = C if m  = y_i,
//  C^m_i = 0 if m != y_i,
//  and w_m(\alpha) = \sum_i \alpha^m_i x_i
//
// Given:
// x, y, C
// eps is the stopping tolerance
//
// solution will be put in w
//
// See Appendix of LIBLINEAR paper, Fan et al. (2008)

#define GETI(i) ((int) prob->y[i])
// To support weights for instances, use GETI(i) (i)

class Solver_MCSVM_CS
{
	public:
		Solver_MCSVM_CS(const problem *prob, int nr_class, double *C, double eps=0.1, int max_iter=100000);
		~Solver_MCSVM_CS();
		void Solve(double *w);
	private:
		void solve_sub_problem(double A_i, int yi, double C_yi, int active_i, double *alpha_new);
		bool be_shrunk(int i, int m, int yi, double alpha_i, double minG);
		double *B, *C, *G;
		int w_size, l;
		int nr_class;
		int max_iter;
		double eps;
		const problem *prob;
};

Solver_MCSVM_CS::Solver_MCSVM_CS(const problem *prob, int nr_class, double *weighted_C, double eps, int max_iter)
{
	this->w_size = prob->n;
	this->l = prob->l;
	this->nr_class = nr_class;
	this->eps = eps;
	this->max_iter = max_iter;
	this->prob = prob;
	this->B = new double[nr_class];
	this->G = new double[nr_class];
	this->C = weighted_C;
}

Solver_MCSVM_CS::~Solver_MCSVM_CS()
{
	delete[] B;
	delete[] G;
}

int compare_double(const void *a, const void *b)
{
	if(*(double *)a > *(double *)b)
		return -1;
	if(*(double *)a < *(double *)b)
		return 1;
	return 0;
}

void Solver_MCSVM_CS::solve_sub_problem(double A_i, int yi, double C_yi, int active_i, double *alpha_new)
{
	int r;
	double *D;

	clone(D, B, active_i);
	if(yi < active_i)
		D[yi] += A_i*C_yi;
	qsort(D, active_i, sizeof(double), compare_double);

	double beta = D[0] - A_i*C_yi;
	for(r=1;r<active_i && beta<r*D[r];r++)
		beta += D[r];
	beta /= r;

	for(r=0;r<active_i;r++)
	{
		if(r == yi)
			alpha_new[r] = min(C_yi, (beta-B[r])/A_i);
		else
			alpha_new[r] = min((double)0, (beta - B[r])/A_i);
	}
	delete[] D;
}

bool Solver_MCSVM_CS::be_shrunk(int i, int m, int yi, double alpha_i, double minG)
{
	double bound = 0;
	if(m == yi)
		bound = C[GETI(i)];
	if(alpha_i == bound && G[m] < minG)
		return true;
	return false;
}

void Solver_MCSVM_CS::Solve(double *w)
{
	int i, m, s;
	int iter = 0;
	double *alpha =  new double[l*nr_class];
	double *alpha_new = new double[nr_class];
	int *index = new int[l];
	double *QD = new double[l];
	int *d_ind = new int[nr_class];
	double *d_val = new double[nr_class];
	int *alpha_index = new int[nr_class*l];
	int *y_index = new int[l];
	int active_size = l;
	int *active_size_i = new int[l];
	double eps_shrink = max(10.0*eps, 1.0); // stopping tolerance for shrinking
	bool start_from_all = true;

	// Initial alpha can be set here. Note that
	// sum_m alpha[i*nr_class+m] = 0, for all i=1,...,l-1
	// alpha[i*nr_class+m] <= C[GETI(i)] if prob->y[i] == m
	// alpha[i*nr_class+m] <= 0 if prob->y[i] != m
	// If initial alpha isn't zero, uncomment the for loop below to initialize w
	for(i=0;i<l*nr_class;i++)
		alpha[i] = 0;

	for(i=0;i<w_size*nr_class;i++)
		w[i] = 0;
	for(i=0;i<l;i++)
	{
		for(m=0;m<nr_class;m++)
			alpha_index[i*nr_class+m] = m;
		feature_node *xi = prob->x[i];
		QD[i] = 0;
		while(xi->index != -1)
		{
			double val = xi->value;
			QD[i] += val*val;

			// Uncomment the for loop if initial alpha isn't zero
			// for(m=0; m<nr_class; m++)
			//	w[(xi->index-1)*nr_class+m] += alpha[i*nr_class+m]*val;
			xi++;
		}
		active_size_i[i] = nr_class;
		y_index[i] = (int)prob->y[i];
		index[i] = i;
	}

	while(iter < max_iter)
	{
		double stopping = -INF;
		for(i=0;i<active_size;i++)
		{
			int j = i+rand()%(active_size-i);
			swap(index[i], index[j]);
		}
		for(s=0;s<active_size;s++)
		{
			i = index[s];
			double Ai = QD[i];
			double *alpha_i = &alpha[i*nr_class];
			int *alpha_index_i = &alpha_index[i*nr_class];

			if(Ai > 0)
			{
				for(m=0;m<active_size_i[i];m++)
					G[m] = 1;
				if(y_index[i] < active_size_i[i])
					G[y_index[i]] = 0;

				feature_node *xi = prob->x[i];
				while(xi->index!= -1)
				{
					double *w_i = &w[(xi->index-1)*nr_class];
					for(m=0;m<active_size_i[i];m++)
						G[m] += w_i[alpha_index_i[m]]*(xi->value);
					xi++;
				}

				double minG = INF;
				double maxG = -INF;
				for(m=0;m<active_size_i[i];m++)
				{
					if(alpha_i[alpha_index_i[m]] < 0 && G[m] < minG)
						minG = G[m];
					if(G[m] > maxG)
						maxG = G[m];
				}
				if(y_index[i] < active_size_i[i])
					if(alpha_i[(int) prob->y[i]] < C[GETI(i)] && G[y_index[i]] < minG)
						minG = G[y_index[i]];

				for(m=0;m<active_size_i[i];m++)
				{
					if(be_shrunk(i, m, y_index[i], alpha_i[alpha_index_i[m]], minG))
					{
						active_size_i[i]--;
						while(active_size_i[i]>m)
						{
							if(!be_shrunk(i, active_size_i[i], y_index[i],
											alpha_i[alpha_index_i[active_size_i[i]]], minG))
							{
								swap(alpha_index_i[m], alpha_index_i[active_size_i[i]]);
								swap(G[m], G[active_size_i[i]]);
								if(y_index[i] == active_size_i[i])
									y_index[i] = m;
								else if(y_index[i] == m)
									y_index[i] = active_size_i[i];
								break;
							}
							active_size_i[i]--;
						}
					}
				}

				if(active_size_i[i] <= 1)
				{
					active_size--;
					swap(index[s], index[active_size]);
					s--;
					continue;
				}

				if(maxG-minG <= 1e-12)
					continue;
				else
					stopping = max(maxG - minG, stopping);

				for(m=0;m<active_size_i[i];m++)
					B[m] = G[m] - Ai*alpha_i[alpha_index_i[m]] ;

				solve_sub_problem(Ai, y_index[i], C[GETI(i)], active_size_i[i], alpha_new);
				int nz_d = 0;
				for(m=0;m<active_size_i[i];m++)
				{
					double d = alpha_new[m] - alpha_i[alpha_index_i[m]];
					alpha_i[alpha_index_i[m]] = alpha_new[m];
					if(fabs(d) >= 1e-12)
					{
						d_ind[nz_d] = alpha_index_i[m];
						d_val[nz_d] = d;
						nz_d++;
					}
				}

				xi = prob->x[i];
				while(xi->index != -1)
				{
					double *w_i = &w[(xi->index-1)*nr_class];
					for(m=0;m<nz_d;m++)
						w_i[d_ind[m]] += d_val[m]*xi->value;
					xi++;
				}
			}
		}

		iter++;
		if(iter % 10 == 0)
		{
			info(".");
		}

		if(stopping < eps_shrink)
		{
			if(stopping < eps && start_from_all == true)
				break;
			else
			{
				active_size = l;
				for(i=0;i<l;i++)
					active_size_i[i] = nr_class;
				info("*");
				eps_shrink = max(eps_shrink/2, eps);
				start_from_all = true;
			}
		}
		else
			start_from_all = false;
	}

	info("\noptimization finished, #iter = %d\n",iter);
	if (iter >= max_iter)
		info("\nWARNING: reaching max number of iterations\n");

	// calculate objective value
	double v = 0;
	int nSV = 0;
	for(i=0;i<w_size*nr_class;i++)
		v += w[i]*w[i];
	v = 0.5*v;
	for(i=0;i<l*nr_class;i++)
	{
		v += alpha[i];
		if(fabs(alpha[i]) > 0)
			nSV++;
	}
	for(i=0;i<l;i++)
		v -= alpha[i*nr_class+(int)prob->y[i]];
	info("Objective value = %lf\n",v);
	info("nSV = %d\n",nSV);

	delete [] alpha;
	delete [] alpha_new;
	delete [] index;
	delete [] QD;
	delete [] d_ind;
	delete [] d_val;
	delete [] alpha_index;
	delete [] y_index;
	delete [] active_size_i;
}

// A coordinate descent algorithm for
// L1-loss and L2-loss SVM dual problems
//
//  min_\alpha  0.5(\alpha^T (Q + D)\alpha) - e^T \alpha,
//    s.t.      0 <= \alpha_i <= upper_bound_i,
//
//  where Qij = yi yj xi^T xj and
//  D is a diagonal matrix
//
// In L1-SVM case:
// 		upper_bound_i = Cp if y_i = 1
// 		upper_bound_i = Cn if y_i = -1
// 		D_ii = 0
// In L2-SVM case:
// 		upper_bound_i = INF
// 		D_ii = 1/(2*Cp)	if y_i = 1
// 		D_ii = 1/(2*Cn)	if y_i = -1
//
// Given:
// x, y, Cp, Cn
// eps is the stopping tolerance
//
// solution will be put in w
//
// See Algorithm 3 of Hsieh et al., ICML 2008

#undef GETI
#define GETI(i) (y[i]+1)
// To support weights for instances, use GETI(i) (i)

static void solve_l2r_l1l2_svc(
	const problem *prob, double *w, double eps,
	double Cp, double Cn, int solver_type)
{
	int l = prob->l;
	int w_size = prob->n;
	int i, s, iter = 0;
	double C, d, G;
	double *QD = new double[l];
	int max_iter = 1000;
	int *index = new int[l];
	double *alpha = new double[l];
	schar *y = new schar[l];
	int active_size = l;

	// PG: projected gradient, for shrinking and stopping
	double PG;
	double PGmax_old = INF;
	double PGmin_old = -INF;
	double PGmax_new, PGmin_new;

	// default solver_type: L2R_L2LOSS_SVC_DUAL
	double diag[3] = {0.5/Cn, 0, 0.5/Cp};
	double upper_bound[3] = {INF, 0, INF};
	if(solver_type == L2R_L1LOSS_SVC_DUAL)
	{
		diag[0] = 0;
		diag[2] = 0;
		upper_bound[0] = Cn;
		upper_bound[2] = Cp;
	}

	for(i=0; i<l; i++)
	{
		if(prob->y[i] > 0)
		{
			y[i] = +1;
		}
		else
		{
			y[i] = -1;
		}
	}

	// Initial alpha can be set here. Note that
	// 0 <= alpha[i] <= upper_bound[GETI(i)]
	for(i=0; i<l; i++)
		alpha[i] = 0;

	for(i=0; i<w_size; i++)
		w[i] = 0;
	for(i=0; i<l; i++)
	{
		QD[i] = diag[GETI(i)];

		feature_node * const xi = prob->x[i];
		QD[i] += sparse_operator::nrm2_sq(xi);
		sparse_operator::axpy(y[i]*alpha[i], xi, w);

		index[i] = i;
	}

	while (iter < max_iter)
	{
		PGmax_new = -INF;
		PGmin_new = INF;

		for (i=0; i<active_size; i++)
		{
			int j = i+rand()%(active_size-i);
			swap(index[i], index[j]);
		}

		for (s=0; s<active_size; s++)
		{
			i = index[s];
			const schar yi = y[i];
			feature_node * const xi = prob->x[i];

			G = yi*sparse_operator::dot(w, xi)-1;

			C = upper_bound[GETI(i)];
			G += alpha[i]*diag[GETI(i)];

			PG = 0;
			if (alpha[i] == 0)
			{
				if (G > PGmax_old)
				{
					active_size--;
					swap(index[s], index[active_size]);
					s--;
					continue;
				}
				else if (G < 0)
					PG = G;
			}
			else if (alpha[i] == C)
			{
				if (G < PGmin_old)
				{
					active_size--;
					swap(index[s], index[active_size]);
					s--;
					continue;
				}
				else if (G > 0)
					PG = G;
			}
			else
				PG = G;

			PGmax_new = max(PGmax_new, PG);
			PGmin_new = min(PGmin_new, PG);

			if(fabs(PG) > 1.0e-12)
			{
				double alpha_old = alpha[i];
				alpha[i] = min(max(alpha[i] - G/QD[i], 0.0), C);
				d = (alpha[i] - alpha_old)*yi;
				sparse_operator::axpy(d, xi, w);
			}
		}

		iter++;
		if(iter % 10 == 0)
			info(".");

		if(PGmax_new - PGmin_new <= eps)
		{
			if(active_size == l)
				break;
			else
			{
				active_size = l;
				info("*");
				PGmax_old = INF;
				PGmin_old = -INF;
				continue;
			}
		}
		PGmax_old = PGmax_new;
		PGmin_old = PGmin_new;
		if (PGmax_old <= 0)
			PGmax_old = INF;
		if (PGmin_old >= 0)
			PGmin_old = -INF;
	}

	info("\noptimization finished, #iter = %d\n",iter);
	if (iter >= max_iter)
		info("\nWARNING: reaching max number of iterations\nUsing -s 2 may be faster (also see FAQ)\n\n");

	// calculate objective value

	double v = 0;
	int nSV = 0;
	for(i=0; i<w_size; i++)
		v += w[i]*w[i];
	for(i=0; i<l; i++)
	{
		v += alpha[i]*(alpha[i]*diag[GETI(i)] - 2);
		if(alpha[i] > 0)
			++nSV;
	}
	info("Objective value = %lf\n",v/2);
	info("nSV = %d\n",nSV);

	delete [] QD;
	delete [] alpha;
	delete [] y;
	delete [] index;
}


// A coordinate descent algorithm for
// L1-loss and L2-loss epsilon-SVR dual problem
//
//  min_\beta  0.5\beta^T (Q + diag(lambda)) \beta - p \sum_{i=1}^l|\beta_i| + \sum_{i=1}^l yi\beta_i,
//    s.t.      -upper_bound_i <= \beta_i <= upper_bound_i,
//
//  where Qij = xi^T xj and
//  D is a diagonal matrix
//
// In L1-SVM case:
// 		upper_bound_i = C
// 		lambda_i = 0
// In L2-SVM case:
// 		upper_bound_i = INF
// 		lambda_i = 1/(2*C)
//
// Given:
// x, y, p, C
// eps is the stopping tolerance
//
// solution will be put in w
//
// See Algorithm 4 of Ho and Lin, 2012

#undef GETI
#define GETI(i) (0)
// To support weights for instances, use GETI(i) (i)

static void solve_l2r_l1l2_svr(
	const problem *prob, double *w, const parameter *param,
	int solver_type)
{
	int l = prob->l;
	double C = param->C;
	double p = param->p;
	int w_size = prob->n;
	double eps = param->eps;
	int i, s, iter = 0;
	int max_iter = 1000;
	int active_size = l;
	int *index = new int[l];

	double d, G, H;
	double Gmax_old = INF;
	double Gmax_new, Gnorm1_new;
	double Gnorm1_init = -1.0; // Gnorm1_init is initialized at the first iteration
	double *beta = new double[l];
	double *QD = new double[l];
	double *y = prob->y;

	// L2R_L2LOSS_SVR_DUAL
	double lambda[1], upper_bound[1];
	lambda[0] = 0.5/C;
	upper_bound[0] = INF;

	if(solver_type == L2R_L1LOSS_SVR_DUAL)
	{
		lambda[0] = 0;
		upper_bound[0] = C;
	}

	// Initial beta can be set here. Note that
	// -upper_bound <= beta[i] <= upper_bound
	for(i=0; i<l; i++)
		beta[i] = 0;

	for(i=0; i<w_size; i++)
		w[i] = 0;
	for(i=0; i<l; i++)
	{
		feature_node * const xi = prob->x[i];
		QD[i] = sparse_operator::nrm2_sq(xi);
		sparse_operator::axpy(beta[i], xi, w);

		index[i] = i;
	}


	while(iter < max_iter)
	{
		Gmax_new = 0;
		Gnorm1_new = 0;

		for(i=0; i<active_size; i++)
		{
			int j = i+rand()%(active_size-i);
			swap(index[i], index[j]);
		}

		for(s=0; s<active_size; s++)
		{
			i = index[s];
			G = -y[i] + lambda[GETI(i)]*beta[i];
			H = QD[i] + lambda[GETI(i)];

			feature_node * const xi = prob->x[i];
			G += sparse_operator::dot(w, xi);

			double Gp = G+p;
			double Gn = G-p;
			double violation = 0;
			if(beta[i] == 0)
			{
				if(Gp < 0)
					violation = -Gp;
				else if(Gn > 0)
					violation = Gn;
				else if(Gp>Gmax_old && Gn<-Gmax_old)
				{
					active_size--;
					swap(index[s], index[active_size]);
					s--;
					continue;
				}
			}
			else if(beta[i] >= upper_bound[GETI(i)])
			{
				if(Gp > 0)
					violation = Gp;
				else if(Gp < -Gmax_old)
				{
					active_size--;
					swap(index[s], index[active_size]);
					s--;
					continue;
				}
			}
			else if(beta[i] <= -upper_bound[GETI(i)])
			{
				if(Gn < 0)
					violation = -Gn;
				else if(Gn > Gmax_old)
				{
					active_size--;
					swap(index[s], index[active_size]);
					s--;
					continue;
				}
			}
			else if(beta[i] > 0)
				violation = fabs(Gp);
			else
				violation = fabs(Gn);

			Gmax_new = max(Gmax_new, violation);
			Gnorm1_new += violation;

			// obtain Newton direction d
			if(Gp < H*beta[i])
				d = -Gp/H;
			else if(Gn > H*beta[i])
				d = -Gn/H;
			else
				d = -beta[i];

			if(fabs(d) < 1.0e-12)
				continue;

			double beta_old = beta[i];
			beta[i] = min(max(beta[i]+d, -upper_bound[GETI(i)]), upper_bound[GETI(i)]);
			d = beta[i]-beta_old;

			if(d != 0)
				sparse_operator::axpy(d, xi, w);
		}

		if(iter == 0)
			Gnorm1_init = Gnorm1_new;
		iter++;
		if(iter % 10 == 0)
			info(".");

		if(Gnorm1_new <= eps*Gnorm1_init)
		{
			if(active_size == l)
				break;
			else
			{
				active_size = l;
				info("*");
				Gmax_old = INF;
				continue;
			}
		}

		Gmax_old = Gmax_new;
	}

	info("\noptimization finished, #iter = %d\n", iter);
	if(iter >= max_iter)
		info("\nWARNING: reaching max number of iterations\nUsing -s 11 may be faster\n\n");

	// calculate objective value
	double v = 0;
	int nSV = 0;
	for(i=0; i<w_size; i++)
		v += w[i]*w[i];
	v = 0.5*v;
	for(i=0; i<l; i++)
	{
		v += p*fabs(beta[i]) - y[i]*beta[i] + 0.5*lambda[GETI(i)]*beta[i]*beta[i];
		if(beta[i] != 0)
			nSV++;
	}

	info("Objective value = %lf\n", v);
	info("nSV = %d\n",nSV);

	delete [] beta;
	delete [] QD;
	delete [] index;
}


// A coordinate descent algorithm for
// the dual of L2-regularized logistic regression problems
//
//  min_\alpha  0.5(\alpha^T Q \alpha) + \sum \alpha_i log (\alpha_i) + (upper_bound_i - \alpha_i) log (upper_bound_i - \alpha_i),
//    s.t.      0 <= \alpha_i <= upper_bound_i,
//
//  where Qij = yi yj xi^T xj and
//  upper_bound_i = Cp if y_i = 1
//  upper_bound_i = Cn if y_i = -1
//
// Given:
// x, y, Cp, Cn
// eps is the stopping tolerance
//
// solution will be put in w
//
// See Algorithm 5 of Yu et al., MLJ 2010

#undef GETI
#define GETI(i) (y[i]+1)
// To support weights for instances, use GETI(i) (i)

void solve_l2r_lr_dual(const problem *prob, double *w, double eps, double Cp, double Cn)
{
	int l = prob->l;
	int w_size = prob->n;
	int i, s, iter = 0;
	double *xTx = new double[l];
	int max_iter = 1000;
	int *index = new int[l];
	double *alpha = new double[2*l]; // store alpha and C - alpha
	schar *y = new schar[l];
	int max_inner_iter = 100; // for inner Newton
	double innereps = 1e-2;
	double innereps_min = min(1e-8, eps);
	double upper_bound[3] = {Cn, 0, Cp};

	for(i=0; i<l; i++)
	{
		if(prob->y[i] > 0)
		{
			y[i] = +1;
		}
		else
		{
			y[i] = -1;
		}
	}

	// Initial alpha can be set here. Note that
	// 0 < alpha[i] < upper_bound[GETI(i)]
	// alpha[2*i] + alpha[2*i+1] = upper_bound[GETI(i)]
	for(i=0; i<l; i++)
	{
		alpha[2*i] = min(0.001*upper_bound[GETI(i)], 1e-8);
		alpha[2*i+1] = upper_bound[GETI(i)] - alpha[2*i];
	}

	for(i=0; i<w_size; i++)
		w[i] = 0;
	for(i=0; i<l; i++)
	{
		feature_node * const xi = prob->x[i];
		xTx[i] = sparse_operator::nrm2_sq(xi);
		sparse_operator::axpy(y[i]*alpha[2*i], xi, w);
		index[i] = i;
	}

	while (iter < max_iter)
	{
		for (i=0; i<l; i++)
		{
			int j = i+rand()%(l-i);
			swap(index[i], index[j]);
		}
		int newton_iter = 0;
		double Gmax = 0;
		for (s=0; s<l; s++)
		{
			i = index[s];
			const schar yi = y[i];
			double C = upper_bound[GETI(i)];
			double ywTx = 0, xisq = xTx[i];
			feature_node * const xi = prob->x[i];
			ywTx = yi*sparse_operator::dot(w, xi);
			double a = xisq, b = ywTx;

			// Decide to minimize g_1(z) or g_2(z)
			int ind1 = 2*i, ind2 = 2*i+1, sign = 1;
			if(0.5*a*(alpha[ind2]-alpha[ind1])+b < 0)
			{
				ind1 = 2*i+1;
				ind2 = 2*i;
				sign = -1;
			}

			//  g_t(z) = z*log(z) + (C-z)*log(C-z) + 0.5a(z-alpha_old)^2 + sign*b(z-alpha_old)
			double alpha_old = alpha[ind1];
			double z = alpha_old;
			if(C - z < 0.5 * C)
				z = 0.1*z;
			double gp = a*(z-alpha_old)+sign*b+log(z/(C-z));
			Gmax = max(Gmax, fabs(gp));

			// Newton method on the sub-problem
			const double eta = 0.1; // xi in the paper
			int inner_iter = 0;
			while (inner_iter <= max_inner_iter)
			{
				if(fabs(gp) < innereps)
					break;
				double gpp = a + C/(C-z)/z;
				double tmpz = z - gp/gpp;
				if(tmpz <= 0)
					z *= eta;
				else // tmpz in (0, C)
					z = tmpz;
				gp = a*(z-alpha_old)+sign*b+log(z/(C-z));
				newton_iter++;
				inner_iter++;
			}

			if(inner_iter > 0) // update w
			{
				alpha[ind1] = z;
				alpha[ind2] = C-z;
				sparse_operator::axpy(sign*(z-alpha_old)*yi, xi, w);
			}
		}

		iter++;
		if(iter % 10 == 0)
			info(".");

		if(Gmax < eps)
			break;

		if(newton_iter <= l/10)
			innereps = max(innereps_min, 0.1*innereps);

	}

	info("\noptimization finished, #iter = %d\n",iter);
	if (iter >= max_iter)
		info("\nWARNING: reaching max number of iterations\nUsing -s 0 may be faster (also see FAQ)\n\n");

	// calculate objective value

	double v = 0;
	for(i=0; i<w_size; i++)
		v += w[i] * w[i];
	v *= 0.5;
	for(i=0; i<l; i++)
		v += alpha[2*i] * log(alpha[2*i]) + alpha[2*i+1] * log(alpha[2*i+1])
			- upper_bound[GETI(i)] * log(upper_bound[GETI(i)]);
	info("Objective value = %lf\n", v);

	delete [] xTx;
	delete [] alpha;
	delete [] y;
	delete [] index;
}

// A coordinate descent algorithm for
// L1-regularized L2-loss support vector classification
//
//  min_w \sum |wj| + C \sum max(0, 1-yi w^T xi)^2,
//
// Given:
// x, y, Cp, Cn
// eps is the stopping tolerance
//
// solution will be put in w
//
// See Yuan et al. (2010) and appendix of LIBLINEAR paper, Fan et al. (2008)

#undef GETI
#define GETI(i) (y[i]+1)
// To support weights for instances, use GETI(i) (i)

static void solve_l1r_l2_svc(
	problem *prob_col, double *w, double eps,
	double Cp, double Cn)
{
	int l = prob_col->l;
	int w_size = prob_col->n;
	int j, s, iter = 0;
	int max_iter = 1000;
	int active_size = w_size;
	int max_num_linesearch = 20;

	double sigma = 0.01;
	double d, G_loss, G, H;
	double Gmax_old = INF;
	double Gmax_new, Gnorm1_new;
	double Gnorm1_init = -1.0; // Gnorm1_init is initialized at the first iteration
	double d_old, d_diff;
	double loss_old, loss_new;
	double appxcond, cond;

	int *index = new int[w_size];
	schar *y = new schar[l];
	double *b = new double[l]; // b = 1-ywTx
	double *xj_sq = new double[w_size];
	feature_node *x;

	double C[3] = {Cn,0,Cp};

	// Initial w can be set here.
	/*
	for(j=0; j<w_size; j++)
		w[j] = 0;
	*/

	for(j=0; j<l; j++)
	{
		b[j] = 1;
		if(prob_col->y[j] > 0)
			y[j] = 1;
		else
			y[j] = -1;
	}
	for(j=0; j<w_size; j++)
	{
		index[j] = j;
		xj_sq[j] = 0;
		x = prob_col->x[j];
		while(x->index != -1)
		{
			int ind = x->index-1;
			x->value *= y[ind]; // x->value stores yi*xij
			double val = x->value;
			b[ind] -= w[j]*val;
			xj_sq[j] += C[GETI(ind)]*val*val;
			x++;
		}
	}

	while(iter < max_iter)
	{
		if(iter % 10 == 0)
		{
			info(".");
			// calculate objective value

			double v = 0;
			int nnz = 0;
			for(j=0; j<w_size; j++)
			{
				x = prob_col->x[j];
				while(x->index != -1)
				{
					x->value *= prob_col->y[x->index-1]; // restore x->value
					x++;
				}
				if(w[j] != 0)
				{
					v += fabs(w[j]);
					nnz++;
				}
			}
			for(j=0; j<l; j++)
				if(b[j] > 0)
					v += C[GETI(j)]*b[j]*b[j];

			info("iter %d Objective value = %lf\n", iter, v);
			info("#nonzeros/#features = %d/%d\n", nnz, w_size);
		}

		Gmax_new = 0;
		Gnorm1_new = 0;

		for(j=0; j<active_size; j++)
		{
			int i = j+rand()%(active_size-j);
			swap(index[i], index[j]);
		}

		for(s=0; s<active_size; s++)
		{
			j = index[s];
			G_loss = 0;
			H = 0;

			x = prob_col->x[j];
			while(x->index != -1)
			{
				int ind = x->index-1;
				if(b[ind] > 0)
				{
					double val = x->value;
					double tmp = C[GETI(ind)]*val;
					G_loss -= tmp*b[ind];
					H += tmp*val;
				}
				x++;
			}
			G_loss *= 2;

			G = G_loss;
			H *= 2;
			H = max(H, 1e-12);

			double Gp = G+1;
			double Gn = G-1;
			double violation = 0;
			if(w[j] == 0)
			{
				if(Gp < 0)
					violation = -Gp;
				else if(Gn > 0)
					violation = Gn;
				else if(Gp>Gmax_old/l && Gn<-Gmax_old/l)
				{
					active_size--;
					swap(index[s], index[active_size]);
					s--;
					continue;
				}
			}
			else if(w[j] > 0)
				violation = fabs(Gp);
			else
				violation = fabs(Gn);

			Gmax_new = max(Gmax_new, violation);
			Gnorm1_new += violation;

			// obtain Newton direction d
			if(Gp < H*w[j])
				d = -Gp/H;
			else if(Gn > H*w[j])
				d = -Gn/H;
			else
				d = -w[j];

			if(fabs(d) < 1.0e-12)
				continue;

			double delta = fabs(w[j]+d)-fabs(w[j]) + G*d;
			d_old = 0;
			int num_linesearch;
			for(num_linesearch=0; num_linesearch < max_num_linesearch; num_linesearch++)
			{
				d_diff = d_old - d;
				cond = fabs(w[j]+d)-fabs(w[j]) - sigma*delta;

				appxcond = xj_sq[j]*d*d + G_loss*d + cond;
				if(appxcond <= 0)
				{
					x = prob_col->x[j];
					sparse_operator::axpy(d_diff, x, b);
					break;
				}

				if(num_linesearch == 0)
				{
					loss_old = 0;
					loss_new = 0;
					x = prob_col->x[j];
					while(x->index != -1)
					{
						int ind = x->index-1;
						if(b[ind] > 0)
							loss_old += C[GETI(ind)]*b[ind]*b[ind];
						double b_new = b[ind] + d_diff*x->value;
						b[ind] = b_new;
						if(b_new > 0)
							loss_new += C[GETI(ind)]*b_new*b_new;
						x++;
					}
				}
				else
				{
					loss_new = 0;
					x = prob_col->x[j];
					while(x->index != -1)
					{
						int ind = x->index-1;
						double b_new = b[ind] + d_diff*x->value;
						b[ind] = b_new;
						if(b_new > 0)
							loss_new += C[GETI(ind)]*b_new*b_new;
						x++;
					}
				}

				cond = cond + loss_new - loss_old;
				if(cond <= 0)
					break;
				else
				{
					d_old = d;
					d *= 0.5;
					delta *= 0.5;
				}
			}

			w[j] += d;

			// recompute b[] if line search takes too many steps
			if(num_linesearch >= max_num_linesearch)
			{
				info("#");
				for(int i=0; i<l; i++)
					b[i] = 1;

				for(int i=0; i<w_size; i++)
				{
					if(w[i]==0) continue;
					x = prob_col->x[i];
					sparse_operator::axpy(-w[i], x, b);
				}
			}
		}

		if(iter == 0)
			Gnorm1_init = Gnorm1_new;
		iter++;

		if(Gnorm1_new <= eps*Gnorm1_init)
		{
			if(active_size == w_size)
				break;
			else
			{
				active_size = w_size;
				info("*");
				Gmax_old = INF;
				continue;
			}
		}

		Gmax_old = Gmax_new;
	}

	info("\noptimization finished, #iter = %d\n", iter);
	if(iter >= max_iter)
		info("\nWARNING: reaching max number of iterations\n");

	// calculate objective value

	double v = 0;
	int nnz = 0;
	for(j=0; j<w_size; j++)
	{
		x = prob_col->x[j];
		while(x->index != -1)
		{
			x->value *= prob_col->y[x->index-1]; // restore x->value
			x++;
		}
		if(w[j] != 0)
		{
			v += fabs(w[j]);
			nnz++;
		}
	}
	for(j=0; j<l; j++)
		if(b[j] > 0)
			v += C[GETI(j)]*b[j]*b[j];

	info("\niter %d Objective value = %lf\n", iter, v);
	info("#nonzeros/#features = %d/%d\n", nnz, w_size);

	delete [] index;
	delete [] y;
	delete [] b;
	delete [] xj_sq;
}

// A coordinate descent algorithm for
// L1-regularized logistic regression problems
//
//  min_w \sum |wj| + C \sum log(1+exp(-yi w^T xi)),
//
// Given:
// x, y, Cp, Cn
// eps is the stopping tolerance
//
// solution will be put in w
//
// See Yuan et al. (2011) and appendix of LIBLINEAR paper, Fan et al. (2008)

#undef GETI
#define GETI(i) (y[i]+1)
// To support weights for instances, use GETI(i) (i)

static void solve_l1r_lr(
	const problem *prob_col, double *w, double eps,
	double Cp, double Cn)
{
	int l = prob_col->l;
	int w_size = prob_col->n;
	int j, s, newton_iter=0, iter=0;
	int max_newton_iter = 100;
	int max_iter = 1000;
	int max_num_linesearch = 20;
	int active_size;
	int QP_active_size;

	double nu = 1e-12;
	double inner_eps = 1;
	double sigma = 0.01;
	double w_norm, w_norm_new;
	double z, G, H;
	double Gnorm1_init = -1.0; // Gnorm1_init is initialized at the first iteration
	double Gmax_old = INF;
	double Gmax_new, Gnorm1_new;
	double QP_Gmax_old = INF;
	double QP_Gmax_new, QP_Gnorm1_new;
	double delta, negsum_xTd, cond;

	int *index = new int[w_size];
	schar *y = new schar[l];
	double *Hdiag = new double[w_size];
	double *Grad = new double[w_size];
	double *wpd = new double[w_size];
	double *xjneg_sum = new double[w_size];
	double *xTd = new double[l];
	double *exp_wTx = new double[l];
	double *exp_wTx_new = new double[l];
	double *tau = new double[l];
	double *D = new double[l];
	feature_node *x;

	double C[3] = {Cn,0,Cp};

	// Initial w can be set here.
	for(j=0; j<w_size; j++)
		w[j] = 0;

	for(j=0; j<l; j++)
	{
		if(prob_col->y[j] > 0)
			y[j] = 1;
		else
			y[j] = -1;

		exp_wTx[j] = 0;
	}

	w_norm = 0;
	for(j=0; j<w_size; j++)
	{
		w_norm += fabs(w[j]);
		wpd[j] = w[j];
		index[j] = j;
		xjneg_sum[j] = 0;
		x = prob_col->x[j];
		while(x->index != -1)
		{
			int ind = x->index-1;
			double val = x->value;
			exp_wTx[ind] += w[j]*val;
			if(y[ind] == -1)
				xjneg_sum[j] += C[GETI(ind)]*val;
			x++;
		}
	}
	for(j=0; j<l; j++)
	{
		exp_wTx[j] = exp(exp_wTx[j]);
		double tau_tmp = 1/(1+exp_wTx[j]);
		tau[j] = C[GETI(j)]*tau_tmp;
		D[j] = C[GETI(j)]*exp_wTx[j]*tau_tmp*tau_tmp;
	}

	while(newton_iter < max_newton_iter)
	{
		Gmax_new = 0;
		Gnorm1_new = 0;
		active_size = w_size;

		for(s=0; s<active_size; s++)
		{
			j = index[s];
			Hdiag[j] = nu;
			Grad[j] = 0;

			double tmp = 0;
			x = prob_col->x[j];
			while(x->index != -1)
			{
				int ind = x->index-1;
				Hdiag[j] += x->value*x->value*D[ind];
				tmp += x->value*tau[ind];
				x++;
			}
			Grad[j] = -tmp + xjneg_sum[j];

			double Gp = Grad[j]+1;
			double Gn = Grad[j]-1;
			double violation = 0;
			if(w[j] == 0)
			{
				if(Gp < 0)
					violation = -Gp;
				else if(Gn > 0)
					violation = Gn;
				//outer-level shrinking
				else if(Gp>Gmax_old/l && Gn<-Gmax_old/l)
				{
					active_size--;
					swap(index[s], index[active_size]);
					s--;
					continue;
				}
			}
			else if(w[j] > 0)
				violation = fabs(Gp);
			else
				violation = fabs(Gn);

			Gmax_new = max(Gmax_new, violation);
			Gnorm1_new += violation;
		}

		if(newton_iter == 0)
			Gnorm1_init = Gnorm1_new;

		if(Gnorm1_new <= eps*Gnorm1_init)
			break;

		iter = 0;
		QP_Gmax_old = INF;
		QP_active_size = active_size;

		for(int i=0; i<l; i++)
			xTd[i] = 0;

		// optimize QP over wpd
		while(iter < max_iter)
		{
			QP_Gmax_new = 0;
			QP_Gnorm1_new = 0;

			for(j=0; j<QP_active_size; j++)
			{
				int i = j+rand()%(QP_active_size-j);
				swap(index[i], index[j]);
			}

			for(s=0; s<QP_active_size; s++)
			{
				j = index[s];
				H = Hdiag[j];

				x = prob_col->x[j];
				G = Grad[j] + (wpd[j]-w[j])*nu;
				while(x->index != -1)
				{
					int ind = x->index-1;
					G += x->value*D[ind]*xTd[ind];
					x++;
				}

				double Gp = G+1;
				double Gn = G-1;
				double violation = 0;
				if(wpd[j] == 0)
				{
					if(Gp < 0)
						violation = -Gp;
					else if(Gn > 0)
						violation = Gn;
					//inner-level shrinking
					else if(Gp>QP_Gmax_old/l && Gn<-QP_Gmax_old/l)
					{
						QP_active_size--;
						swap(index[s], index[QP_active_size]);
						s--;
						continue;
					}
				}
				else if(wpd[j] > 0)
					violation = fabs(Gp);
				else
					violation = fabs(Gn);

				QP_Gmax_new = max(QP_Gmax_new, violation);
				QP_Gnorm1_new += violation;

				// obtain solution of one-variable problem
				if(Gp < H*wpd[j])
					z = -Gp/H;
				else if(Gn > H*wpd[j])
					z = -Gn/H;
				else
					z = -wpd[j];

				if(fabs(z) < 1.0e-12)
					continue;
				z = min(max(z,-10.0),10.0);

				wpd[j] += z;

				x = prob_col->x[j];
				sparse_operator::axpy(z, x, xTd);
			}

			iter++;

			if(QP_Gnorm1_new <= inner_eps*Gnorm1_init)
			{
				//inner stopping
				if(QP_active_size == active_size)
					break;
				//active set reactivation
				else
				{
					QP_active_size = active_size;
					QP_Gmax_old = INF;
					continue;
				}
			}

			QP_Gmax_old = QP_Gmax_new;
		}

		if(iter >= max_iter)
			info("WARNING: reaching max number of inner iterations\n");

		delta = 0;
		w_norm_new = 0;
		for(j=0; j<w_size; j++)
		{
			delta += Grad[j]*(wpd[j]-w[j]);
			if(wpd[j] != 0)
				w_norm_new += fabs(wpd[j]);
		}
		delta += (w_norm_new-w_norm);

		negsum_xTd = 0;
		for(int i=0; i<l; i++)
			if(y[i] == -1)
				negsum_xTd += C[GETI(i)]*xTd[i];

		int num_linesearch;
		for(num_linesearch=0; num_linesearch < max_num_linesearch; num_linesearch++)
		{
			cond = w_norm_new - w_norm + negsum_xTd - sigma*delta;

			for(int i=0; i<l; i++)
			{
				double exp_xTd = exp(xTd[i]);
				exp_wTx_new[i] = exp_wTx[i]*exp_xTd;
				cond += C[GETI(i)]*log((1+exp_wTx_new[i])/(exp_xTd+exp_wTx_new[i]));
			}

			if(cond <= 0)
			{
				w_norm = w_norm_new;
				for(j=0; j<w_size; j++)
					w[j] = wpd[j];
				for(int i=0; i<l; i++)
				{
					exp_wTx[i] = exp_wTx_new[i];
					double tau_tmp = 1/(1+exp_wTx[i]);
					tau[i] = C[GETI(i)]*tau_tmp;
					D[i] = C[GETI(i)]*exp_wTx[i]*tau_tmp*tau_tmp;
				}
				break;
			}
			else
			{
				w_norm_new = 0;
				for(j=0; j<w_size; j++)
				{
					wpd[j] = (w[j]+wpd[j])*0.5;
					if(wpd[j] != 0)
						w_norm_new += fabs(wpd[j]);
				}
				delta *= 0.5;
				negsum_xTd *= 0.5;
				for(int i=0; i<l; i++)
					xTd[i] *= 0.5;
			}
		}

		// Recompute some info due to too many line search steps
		if(num_linesearch >= max_num_linesearch)
		{
			for(int i=0; i<l; i++)
				exp_wTx[i] = 0;

			for(int i=0; i<w_size; i++)
			{
				if(w[i]==0) continue;
				x = prob_col->x[i];
				sparse_operator::axpy(w[i], x, exp_wTx);
			}

			for(int i=0; i<l; i++)
				exp_wTx[i] = exp(exp_wTx[i]);
		}

		if(iter == 1)
			inner_eps *= 0.25;

		newton_iter++;
		Gmax_old = Gmax_new;

		double v = 0;
		int nnz = 0;
		for(j=0; j<w_size; j++)
			if(w[j] != 0)
			{
				v += fabs(w[j]);
				nnz++;
			}
		for(j=0; j<l; j++)
			if(y[j] == 1)
				v += C[GETI(j)]*log(1+1/exp_wTx[j]);
			else
				v += C[GETI(j)]*log(1+exp_wTx[j]);

		info("iter %3d  #CD cycles %d  f %lf  nnz %d\n", newton_iter, iter, v, nnz);
	}

	info("=========================\n");
	info("optimization finished, #iter = %d\n", newton_iter);
	if(newton_iter >= max_newton_iter)
		info("WARNING: reaching max number of iterations\n");

	// calculate objective value

	double v = 0;
	int nnz = 0;
	for(j=0; j<w_size; j++)
		if(w[j] != 0)
		{
			v += fabs(w[j]);
			nnz++;
		}
	for(j=0; j<l; j++)
		if(y[j] == 1)
			v += C[GETI(j)]*log(1+1/exp_wTx[j]);
		else
			v += C[GETI(j)]*log(1+exp_wTx[j]);

	info("Objective value = %lf\n", v);
	info("#nonzeros/#features = %d/%d\n", nnz, w_size);

	delete [] index;
	delete [] y;
	delete [] Hdiag;
	delete [] Grad;
	delete [] wpd;
	delete [] xjneg_sum;
	delete [] xTd;
	delete [] exp_wTx;
	delete [] exp_wTx_new;
	delete [] tau;
	delete [] D;
}

// transpose matrix X from row format to column format
static void transpose(const problem *prob, feature_node **x_space_ret, problem *prob_col)
{
	int i;
	int l = prob->l;
	int n = prob->n;
	size_t nnz = 0;
	size_t *col_ptr = new size_t [n+1];
	feature_node *x_space;
	prob_col->l = l;
	prob_col->n = n;
	prob_col->y = new double[l];
	prob_col->x = new feature_node*[n];

	for(i=0; i<l; i++)
		prob_col->y[i] = prob->y[i];

	for(i=0; i<n+1; i++)
		col_ptr[i] = 0;
	for(i=0; i<l; i++)
	{
		feature_node *x = prob->x[i];
		while(x->index != -1)
		{
			nnz++;
			col_ptr[x->index]++;
			x++;
		}
	}
	for(i=1; i<n+1; i++)
		col_ptr[i] += col_ptr[i-1] + 1;

	x_space = new feature_node[nnz+n];
	for(i=0; i<n; i++)
		prob_col->x[i] = &x_space[col_ptr[i]];

	for(i=0; i<l; i++)
	{
		feature_node *x = prob->x[i];
		while(x->index != -1)
		{
			int ind = x->index-1;
			x_space[col_ptr[ind]].index = i+1; // starts from 1
			x_space[col_ptr[ind]].value = x->value;
			col_ptr[ind]++;
			x++;
		}
	}
	for(i=0; i<n; i++)
		x_space[col_ptr[i]].index = -1;

	*x_space_ret = x_space;

	delete [] col_ptr;
}

// label: label name, start: begin of each class, count: #data of classes, perm: indices to the original data
// perm, length l, must be allocated before calling this subroutine
static void group_classes(const problem *prob, int *nr_class_ret, int **label_ret, int **start_ret, int **count_ret, int *perm)
{
	int l = prob->l;
	int i;

	std::set<int> label_set;
	for(i=0;i<prob->l;i++)
		label_set.insert((int)prob->y[i]);
	
	int label_size = (int)label_set.size();
	int num_machines = mpi_get_size();
	int max_label_size;
	MPI_Allreduce(&label_size, &max_label_size, 1, MPI_INT, MPI_MAX, MPI_COMM_WORLD);

	std::vector<int> global_label_sets((max_label_size+1)*num_machines);
	std::vector<int> label_buff(max_label_size+1);

	label_buff[0] = label_size;
	i = 1;
	for(std::set<int>::iterator this_label=label_set.begin();
			this_label!=label_set.end(); this_label++)
	{
		label_buff[i] = (*this_label);
		i++;
	}
	
	MPI_Allgather(label_buff.data(), max_label_size+1, MPI_INT, global_label_sets.data(), max_label_size+1, MPI_INT, MPI_COMM_WORLD);

	for(i=0; i<num_machines; i++)
	{
		int offset = i*(max_label_size+1);
		int size = global_label_sets[offset];
		for(int j=1; j<=size; j++)
			label_set.insert(global_label_sets[offset+j]);
	}

	int nr_class = (int)label_set.size();

	std::map<int, int> label_map;
	int *label = Malloc(int, nr_class);
	i = 0;
	for(std::set<int>::iterator this_label=label_set.begin();
			this_label!=label_set.end(); this_label++)
	{
		label[i] = (*this_label);
		i++;
	}
	if (nr_class == 2 && label[0] == -1 && label[1] == 1)
		swap(label[0], label[1]);
	for(i=0;i<nr_class;i++)
		label_map[label[i]] = i;


	// The following codes are similar to the original LIBLINEAR
	int *start = Malloc(int, nr_class);
	int *count = Malloc(int, nr_class);
	for(i=0;i<nr_class;i++)
		count[i] = 0;
	for(i=0;i<prob->l;i++)
		count[label_map[(int)prob->y[i]]]++;

	start[0] = 0;
	for(i=1;i<nr_class;i++)
		start[i] = start[i-1]+count[i-1];
	for(i=0;i<l;i++)
	{
		perm[start[label_map[(int)prob->y[i]]]] = i;
		++start[label_map[(int)prob->y[i]]];
	}
	start[0] = 0;
	for(i=1;i<nr_class;i++)
		start[i] = start[i-1]+count[i-1];

	*nr_class_ret = nr_class;
	*label_ret = label;
	*start_ret = start;
	*count_ret = count;
}

static void train_one(const problem *prob, const parameter *param, double *w, double Cp, double Cn)
{
	//inner and outer tolerances for TRON
	double eps = param->eps;
	double eps_cg = 0.1;

	if(param->init_sol != NULL)
		eps_cg = 0.5;

	int l = prob->l;
	int pos = 0;
	int neg = 0;
	for(int i=0;i<l;i++)
		if(prob->y[i] > 0)
			pos++;
	mpi_allreduce(&pos, 1, MPI_INT, MPI_SUM);
	mpi_allreduce(&l, 1, MPI_INT, MPI_SUM);
	neg = l - pos;

	double primal_solver_tol = (eps*max(min(pos,neg), 1))/l;

	function *fun_obj=NULL;

	switch(param->solver_type)
	{
		case L2R_LR:
		{
			double *C = new double[prob->l];
			for(int i = 0; i < prob->l; i++)
			{
				if(prob->y[i] > 0)
					C[i] = Cp;
				else
					C[i] = Cn;
			}
			fun_obj=new l2r_lr_fun(prob, C);
			TRON tron_obj(fun_obj, primal_solver_tol, eps_cg);
			tron_obj.set_print_string(liblinear_print_string);
			tron_obj.tron(w);
			delete fun_obj;
			delete[] C;
			break;
		}
		case L2R_L2LOSS_SVC:
		{
			double *C = new double[prob->l];
			for(int i = 0; i < prob->l; i++)
			{
				if(prob->y[i] > 0)
					C[i] = Cp;
				else
					C[i] = Cn;
			}
			fun_obj=new l2r_l2_svc_fun(prob, C);
			TRON tron_obj(fun_obj, primal_solver_tol, eps_cg);
			tron_obj.set_print_string(liblinear_print_string);
			tron_obj.tron(w);
			delete fun_obj;
			delete[] C;
			break;
		}
		case L2R_L2LOSS_SVC_DUAL:
			solve_l2r_l1l2_svc(prob, w, eps, Cp, Cn, L2R_L2LOSS_SVC_DUAL);
			break;
		case L2R_L1LOSS_SVC_DUAL:
			solve_l2r_l1l2_svc(prob, w, eps, Cp, Cn, L2R_L1LOSS_SVC_DUAL);
			break;
		case L1R_L2LOSS_SVC:
		{
			problem prob_col;
			feature_node *x_space = NULL;
			transpose(prob, &x_space ,&prob_col);
			solve_l1r_l2_svc(&prob_col, w, primal_solver_tol, Cp, Cn);
			delete [] prob_col.y;
			delete [] prob_col.x;
			delete [] x_space;
			break;
		}
		case L1R_LR:
		{
			problem prob_col;
			feature_node *x_space = NULL;
			transpose(prob, &x_space ,&prob_col);
			solve_l1r_lr(&prob_col, w, primal_solver_tol, Cp, Cn);
			delete [] prob_col.y;
			delete [] prob_col.x;
			delete [] x_space;
			break;
		}
		case L2R_LR_DUAL:
		{
			solve_l2r_lr_dual(prob, w, eps, Cp, Cn);
			break;
		}
		case L2R_LR_LBFGS:
		{
			double *C = new double[prob->l];
			for(int i = 0; i < prob->l; i++)
			{
				if(prob->y[i] > 0)
					C[i] = Cp;
				else
					C[i] = Cn;
			}
			fun_obj=new l2r_lr_fun(prob, C);
			LBFGS lbfgs_obj(fun_obj, primal_solver_tol, param->m, param->eta);
			lbfgs_obj.set_print_string(liblinear_print_string);
			lbfgs_obj.lbfgs(w);
			delete fun_obj;
			delete[] C;
			break;
		}
		case L2R_L2LOSS_SVR:
		{
			double *C = new double[prob->l];
			for(int i = 0; i < prob->l; i++)
				C[i] = param->C;

			fun_obj=new l2r_l2_svr_fun(prob, C, param->p);
			TRON tron_obj(fun_obj, param->eps);
			tron_obj.set_print_string(liblinear_print_string);
			tron_obj.tron(w);
			delete fun_obj;
			delete[] C;
			break;

		}
		case L2R_L1LOSS_SVR_DUAL:
			solve_l2r_l1l2_svr(prob, w, param, L2R_L1LOSS_SVR_DUAL);
			break;
		case L2R_L2LOSS_SVR_DUAL:
			solve_l2r_l1l2_svr(prob, w, param, L2R_L2LOSS_SVR_DUAL);
			break;
		case L1R_LR_OWLQN:
		case L1R_LR_OWLQN_COL:
		case L1R_LR_OWLQN_VF:
		case L1R_LR_OWLQN_VF_COL:
		case L1R_LR_NEWTON:
		case L1R_LR_NEWTON_COL:
		case L1R_LR_PROJGRAD:
		case L1R_LR_PROJGRAD_COL:
		case L1R_LR_LCOMM_PG:			
		case L1R_LR_LCOMM_PG_COL:
		case L1R_LR_LCOMM_STEP:
		case L1R_LR_LCOMM_STEP_COL:
		case L1R_LR_LCOMM_LBFGS:
		case L1R_LR_LCOMM_LBFGS_COL:
		{
			double *C = new double[prob->l];
			for(int i = 0; i < prob->l; i++)
			{
				if(prob->y[i] > 0)
					C[i] = Cp;
				else
					C[i] = Cn;
			}
			l1r_lr_fun *l1r_fun_obj=NULL;
			problem prob_col;
			feature_node *x_space = NULL;
			if (param->solver_type == L1R_LR_OWLQN_COL ||
			    param->solver_type == L1R_LR_OWLQN_VF_COL ||
			    param->solver_type == L1R_LR_NEWTON_COL ||
			    param->solver_type == L1R_LR_PROJGRAD_COL ||
			    param->solver_type == L1R_LR_LCOMM_PG_COL ||
			    param->solver_type == L1R_LR_LCOMM_STEP_COL ||
			    param->solver_type == L1R_LR_LCOMM_LBFGS_COL)
			{
				transpose(prob, &x_space ,&prob_col);
				l1r_fun_obj = new l1r_lr_fun_col(&prob_col, C);
			}
			else
				l1r_fun_obj = new l1r_lr_fun_row(prob, C);
			switch (param->solver_type)
			{
				case L1R_LR_OWLQN:
				case L1R_LR_OWLQN_COL:
				case L1R_LR_OWLQN_VF:
				case L1R_LR_OWLQN_VF_COL:
				{
					OWLQN owlqn_obj(l1r_fun_obj, primal_solver_tol, param->m, param->eta);
					owlqn_obj.set_print_string(liblinear_print_string);
					owlqn_obj.owlqn(w,param->solver_type);
					break;
				}
				case L1R_LR_LCOMM_PG:
				case L1R_LR_LCOMM_PG_COL:
				case L1R_LR_LCOMM_STEP:
				case L1R_LR_LCOMM_STEP_COL:
				case L1R_LR_LCOMM_LBFGS:
				case L1R_LR_LCOMM_LBFGS_COL:
				{
					L1_LCOMM lcomm_obj(l1r_fun_obj, primal_solver_tol, param->m, param->eta);
					lcomm_obj.set_print_string(liblinear_print_string);
					lcomm_obj.lcomm(w, param->solver_type);
					break;
				}
				case L1R_LR_NEWTON:
				case L1R_LR_NEWTON_COL:
				{
					L1_NEWTON newton_obj(l1r_fun_obj, primal_solver_tol, param->m, param->eta);
					newton_obj.set_print_string(liblinear_print_string);
					newton_obj.newton(w);
					break;
				}
				case L1R_LR_PROJGRAD:
				case L1R_LR_PROJGRAD_COL:
				{
					PROJGRAD projgrad_obj(l1r_fun_obj, primal_solver_tol, param->m, param->eta);
					projgrad_obj.set_print_string(liblinear_print_string);
					projgrad_obj.projgrad(w);
					break;
				}
			}
			delete l1r_fun_obj;
			delete[] C;

			if (param->solver_type == L1R_LR_OWLQN_COL ||
			    param->solver_type == L1R_LR_OWLQN_VF_COL ||
			    param->solver_type == L1R_LR_NEWTON_COL ||
			    param->solver_type == L1R_LR_PROJGRAD_COL ||
			    param->solver_type == L1R_LR_LCOMM_PG_COL ||
			    param->solver_type == L1R_LR_LCOMM_STEP_COL ||
			    param->solver_type == L1R_LR_LCOMM_LBFGS_COL)
			{
				delete [] prob_col.y;
				delete [] prob_col.x;
				delete [] x_space;
			}
			break;
		}

		default:
			if(mpi_get_rank() == 0)
				fprintf(stderr, "ERROR: unknown solver_type\n");
	}
}

// Calculate the initial C for parameter selection
static double calc_start_C(const problem *prob, const parameter *param)
{
	int i;
	double xTx,max_xTx;
	max_xTx = 0;
	for(i=0; i<prob->l; i++)
	{
		xTx = 0;
		feature_node *xi=prob->x[i];
		while(xi->index != -1)
		{
			double val = xi->value;
			xTx += val*val;
			xi++;
		}
		if(xTx > max_xTx)
			max_xTx = xTx;
	}

	double min_C = 1.0;
	if(param->solver_type == L2R_LR)
		min_C = 1.0 / (prob->l * max_xTx);
	else if(param->solver_type == L2R_L2LOSS_SVC)
		min_C = 1.0 / (2 * prob->l * max_xTx);

	return pow( 2, floor(log(min_C) / log(2.0)) );
}


//
// Interface functions
//
model* train(const problem *prob, const parameter *param)
{
	int i,j;
	int l = prob->l;
	int n = prob->n;
	int w_size = prob->n;
	model *model_ = Malloc(model,1);

	if(prob->bias>=0)
		model_->nr_feature=n-1;
	else
		model_->nr_feature=n;
	model_->param = *param;
	model_->bias = prob->bias;

	if(check_regression_model(model_))
	{
		model_->w = Malloc(double, w_size);
		for(i=0; i<w_size; i++)
			model_->w[i] = 0;
		model_->nr_class = 2;
		model_->label = NULL;
		train_one(prob, param, model_->w, 0, 0);
	}
	else
	{
		int nr_class;
		int *label = NULL;
		int *start = NULL;
		int *count = NULL;
		int *perm = Malloc(int,l);

		// group training data of the same class
		group_classes(prob,&nr_class,&label,&start,&count,perm);

		model_->nr_class=nr_class;
		model_->label = Malloc(int,nr_class);
		for(i=0;i<nr_class;i++)
			model_->label[i] = label[i];

		// calculate weighted C
		double *weighted_C = Malloc(double, nr_class);
		for(i=0;i<nr_class;i++)
			weighted_C[i] = param->C;
		for(i=0;i<param->nr_weight;i++)
		{
			for(j=0;j<nr_class;j++)
				if(param->weight_label[i] == label[j])
					break;
			if(j == nr_class)
			{
				fprintf(stderr,"[rank %d] WARNING: class label %d specified in weight is not found\n",
					mpi_get_rank(), param->weight_label[i]);
			}
			else
				weighted_C[j] *= param->weight[i];
		}

		// constructing the subproblem
		feature_node **x = Malloc(feature_node *,l);
		for(i=0;i<l;i++)
			x[i] = prob->x[perm[i]];

		int k;
		problem sub_prob;
		sub_prob.l = l;
		sub_prob.n = n;
		sub_prob.x = Malloc(feature_node *,sub_prob.l);
		sub_prob.y = Malloc(double,sub_prob.l);

		for(k=0; k<sub_prob.l; k++)
			sub_prob.x[k] = x[k];

		// multi-class svm by Crammer and Singer
		if(param->solver_type == MCSVM_CS)
		{
			model_->w=Malloc(double, n*nr_class);
			for(i=0;i<nr_class;i++)
				for(j=start[i];j<start[i]+count[i];j++)
					sub_prob.y[j] = i;
			Solver_MCSVM_CS Solver(&sub_prob, nr_class, weighted_C, param->eps);
			Solver.Solve(model_->w);
		}
		else
		{
			if(nr_class == 2)
			{
				model_->w=Malloc(double, w_size);

				int e0 = start[0]+count[0];
				k=0;
				for(; k<e0; k++)
					sub_prob.y[k] = +1;
				for(; k<sub_prob.l; k++)
					sub_prob.y[k] = -1;

				if(param->init_sol != NULL)
					for(i=0;i<w_size;i++)
						model_->w[i] = param->init_sol[i];
				else
					for(i=0;i<w_size;i++)
						model_->w[i] = 0;

				train_one(&sub_prob, param, model_->w, weighted_C[0], weighted_C[1]);
			}
			else
			{
				model_->w=Malloc(double, w_size*nr_class);
				double *w=Malloc(double, w_size);
				for(i=0;i<nr_class;i++)
				{
					int si = start[i];
					int ei = si+count[i];

					k=0;
					for(; k<si; k++)
						sub_prob.y[k] = -1;
					for(; k<ei; k++)
						sub_prob.y[k] = +1;
					for(; k<sub_prob.l; k++)
						sub_prob.y[k] = -1;

					if(param->init_sol != NULL)
						for(j=0;j<w_size;j++)
							w[j] = param->init_sol[j*nr_class+i];
					else
						for(j=0;j<w_size;j++)
							w[j] = 0;

					train_one(&sub_prob, param, w, weighted_C[i], param->C);

					for(j=0;j<w_size;j++)
						model_->w[j*nr_class+i] = w[j];
				}
				free(w);
			}

		}

		free(x);
		free(label);
		free(start);
		free(count);
		free(perm);
		free(sub_prob.x);
		free(sub_prob.y);
		free(weighted_C);
	}
	return model_;
}

void cross_validation(const problem *prob, const parameter *param, int nr_fold, double *target)
{
	int i;
	int *fold_start;
	int l = prob->l;
	int *perm = Malloc(int,l);
	if (nr_fold > l)
	{
		nr_fold = l;
		fprintf(stderr,"[rank %d] WARNING: # folds > # data. Will use # folds = # data instead (i.e., leave-one-out cross validation)\n",
		        mpi_get_rank());
	}
	mpi_allreduce(&nr_fold, 1, MPI_INT, MPI_MIN);
	fold_start = Malloc(int,nr_fold+1);
	for(i=0;i<l;i++) perm[i]=i;
	for(i=0;i<l;i++)
	{
		int j = i+rand()%(l-i);
		swap(perm[i],perm[j]);
	}
	for(i=0;i<=nr_fold;i++)
		fold_start[i]=i*l/nr_fold;

	for(i=0;i<nr_fold;i++)
	{
		int begin = fold_start[i];
		int end = fold_start[i+1];
		int j,k;
		struct problem subprob;

		subprob.bias = prob->bias;
		subprob.n = prob->n;
		subprob.l = l-(end-begin);
		subprob.x = Malloc(struct feature_node*,subprob.l);
		subprob.y = Malloc(double,subprob.l);

		k=0;
		for(j=0;j<begin;j++)
		{
			subprob.x[k] = prob->x[perm[j]];
			subprob.y[k] = prob->y[perm[j]];
			++k;
		}
		for(j=end;j<l;j++)
		{
			subprob.x[k] = prob->x[perm[j]];
			subprob.y[k] = prob->y[perm[j]];
			++k;
		}
		struct model *submodel = train(&subprob,param);
		for(j=begin;j<end;j++)
			target[perm[j]] = predict(submodel,prob->x[perm[j]]);
		free_and_destroy_model(&submodel);
		free(subprob.x);
		free(subprob.y);
	}
	free(fold_start);
	free(perm);
}

void find_parameter_C(const problem *prob, const parameter *param, int nr_fold, double start_C, double max_C, double *best_C, double *best_rate)
{
	// variables for CV
	int i;
	int *fold_start;
	int l = prob->l;
	int *perm = Malloc(int, l);
	double *target = Malloc(double, prob->l);
	struct problem *subprob = Malloc(problem,nr_fold);

	// variables for warm start
	double ratio = 2;
	double **prev_w = Malloc(double*, nr_fold);
	for(i = 0; i < nr_fold; i++)
		prev_w[i] = NULL;
	int num_unchanged_w = 0;
	struct parameter param1 = *param;
	void (*default_print_string) (const char *) = liblinear_print_string;

	if (nr_fold > l)
	{
		nr_fold = l;
		fprintf(stderr,"WARNING: # folds > # data. Will use # folds = # data instead (i.e., leave-one-out cross validation)\n");
	}
	fold_start = Malloc(int,nr_fold+1);
	for(i=0;i<l;i++) perm[i]=i;
	for(i=0;i<l;i++)
	{
		int j = i+rand()%(l-i);
		swap(perm[i],perm[j]);
	}
	for(i=0;i<=nr_fold;i++)
		fold_start[i]=i*l/nr_fold;

	for(i=0;i<nr_fold;i++)
	{
		int begin = fold_start[i];
		int end = fold_start[i+1];
		int j,k;

		subprob[i].bias = prob->bias;
		subprob[i].n = prob->n;
		subprob[i].l = l-(end-begin);
		subprob[i].x = Malloc(struct feature_node*,subprob[i].l);
		subprob[i].y = Malloc(double,subprob[i].l);

		k=0;
		for(j=0;j<begin;j++)
		{
			subprob[i].x[k] = prob->x[perm[j]];
			subprob[i].y[k] = prob->y[perm[j]];
			++k;
		}
		for(j=end;j<l;j++)
		{
			subprob[i].x[k] = prob->x[perm[j]];
			subprob[i].y[k] = prob->y[perm[j]];
			++k;
		}

	}

	*best_rate = 0;
	if(start_C <= 0)
		start_C = calc_start_C(prob,param);
	param1.C = start_C;

	while(param1.C <= max_C)
	{
		//Output disabled for running CV at a particular C
		set_print_string_function(&print_null);

		for(i=0; i<nr_fold; i++)
		{
			int j;
			int begin = fold_start[i];
			int end = fold_start[i+1];

			param1.init_sol = prev_w[i];
			struct model *submodel = train(&subprob[i],&param1);

			int total_w_size;
			if(submodel->nr_class == 2)
				total_w_size = subprob[i].n;
			else
				total_w_size = subprob[i].n * submodel->nr_class;

			if(prev_w[i] == NULL)
			{
				prev_w[i] = Malloc(double, total_w_size);
				for(j=0; j<total_w_size; j++)
					prev_w[i][j] = submodel->w[j];
			}
			else if(num_unchanged_w >= 0)
			{
				double norm_w_diff = 0;
				for(j=0; j<total_w_size; j++)
				{
					norm_w_diff += (submodel->w[j] - prev_w[i][j])*(submodel->w[j] - prev_w[i][j]);
					prev_w[i][j] = submodel->w[j];
				}
				norm_w_diff = sqrt(norm_w_diff);

				if(norm_w_diff > 1e-15)
					num_unchanged_w = -1;
			}
			else
			{
				for(j=0; j<total_w_size; j++)
					prev_w[i][j] = submodel->w[j];
			}

			for(j=begin; j<end; j++)
				target[perm[j]] = predict(submodel,prob->x[perm[j]]);

			free_and_destroy_model(&submodel);
		}
		set_print_string_function(default_print_string);

		int total_correct = 0;
		for(i=0; i<prob->l; i++)
			if(target[i] == prob->y[i])
				++total_correct;
		double current_rate = (double)total_correct/prob->l;
		if(current_rate > *best_rate)
		{
			*best_C = param1.C;
			*best_rate = current_rate;
		}

		info("log2c=%7.2f\trate=%g\n",log(param1.C)/log(2.0),100.0*current_rate);
		num_unchanged_w++;
		if(num_unchanged_w == 3)
			break;
		param1.C = param1.C*ratio;
	}

	if(param1.C > max_C && max_C > start_C)
		info("warning: maximum C reached.\n");
	free(fold_start);
	free(perm);
	free(target);
	for(i=0; i<nr_fold; i++)
	{
		free(subprob[i].x);
		free(subprob[i].y);
		free(prev_w[i]);
	}
	free(prev_w);
	free(subprob);
}

void warm_start(const problem *prob, const parameter *param, int nr_fold, double eps_per_split, int solver_per_split, void (*print_func_per_split) (const char*), char *model_file_name)
{

	// variables for CV
	int i;
	int *fold_start;
	int l = prob->l;
	int *perm = Malloc(int, l);

	// variables for warm start
	double **w_vectors = Malloc(double*, nr_fold);
	for(i = 0; i < nr_fold; i++)
		w_vectors[i] = NULL;
	struct parameter param1 = *param;
	param1.C *= nr_fold;
	param1.init_sol = NULL;
	param1.solver_type = solver_per_split;
	param1.eps = eps_per_split;

	void (*default_print_string) (const char *) = liblinear_print_string;
	set_print_string_function(print_func_per_split);

	if (nr_fold > l)
	{
		nr_fold = l;
		fprintf(stderr,"WARNING: # folds > # data. Will use # folds = # data instead (i.e., leave-one-out cross validation)\n");
	}

	fold_start = Malloc(int,nr_fold+1);
	for(i=0;i<l;i++) perm[i]=i;
	for(i=0;i<l;i++)
	{
		int j = i+rand()%(l-i);
		swap(perm[i],perm[j]);
	}
	for(i=0;i<=nr_fold;i++)
		fold_start[i]=i*l/nr_fold;

	for(i=0;i<nr_fold;i++)
	{
		int begin = fold_start[i];
		int end = fold_start[i+1];
		int j,k;
		struct problem subprob;

		subprob.bias = prob->bias;
		subprob.l = end-begin;
		subprob.x = Malloc(struct feature_node*,subprob.l);
		subprob.y = Malloc(double,subprob.l);

		k=0;
		int max_index = 0;
		for(j=begin;j<end;j++)
		{
			subprob.x[k] = prob->x[perm[j]];
			subprob.y[k] = prob->y[perm[j]];


			feature_node *xx = subprob.x[k];
			int inst_max_index = 0;
			while (xx->index != -1) {
				inst_max_index = xx->index;
				xx++;
			}
			if (inst_max_index > max_index)
				max_index = inst_max_index;

			++k;
		}
		mpi_allreduce(&max_index, 1, MPI_INT, MPI_MAX);
		subprob.n = max_index;

		info("\nwarm start (%d/%d) begins!\n", i + 1, nr_fold);
		struct model *submodel = train(&subprob,&param1);

		int total_w_size;
		if(submodel->nr_class == 2)
			total_w_size = subprob.n;
		else
			total_w_size = subprob.n * submodel->nr_class;

		w_vectors[i] = Malloc(double, prob->n);
		for(j=0; j<total_w_size; j++)
			w_vectors[i][j] = submodel->w[j];
		for(j=total_w_size; j<prob->n; j++)
			w_vectors[i][j] = 0;

		free_and_destroy_model(&submodel);
		free(subprob.x);
		free(subprob.y);
	}

	free(fold_start);
	free(perm);

	set_print_string_function(default_print_string);

	double *init_sol=Malloc(double, prob->n);
	for(int j=0; j<prob->n; j++)
		init_sol[j] = 0;
	for(i=0; i<nr_fold; i++)
		for(int j=0; j<prob->n; j++)
			init_sol[j] += w_vectors[i][j];
	for(int j=0; j<prob->n; j++)
		init_sol[j] /= (double) nr_fold;
	param1.init_sol = init_sol;

	for(i=0; i<nr_fold; i++)
		free(w_vectors[i]);
	free(w_vectors);

	param1.C = param->C;
	param1.solver_type = param->solver_type;
	param1.eps = param->eps;

	info("\n**with initialization\n");
	struct model *model = train(prob,&param1);
	save_model(model_file_name, model);
	free_and_destroy_model(&model);
	free(init_sol);

	param1.init_sol = NULL;
	info("\n**no initialization\n");
	model = train(prob,&param1);
	free_and_destroy_model(&model);
}


void warm_start2(const problem *prob, const parameter *param, int nr_fold)
{

	// variables for CV
	int i;
	int *fold_start;
	int l = prob->l;
	int *perm = Malloc(int, l);

	// variables for warm start
	double **w_vectors = Malloc(double*, nr_fold);
	for(i = 0; i < nr_fold; i++)
		w_vectors[i] = NULL;
	struct parameter param1 = *param;
	void (*default_print_string) (const char *) = liblinear_print_string;

	if (nr_fold > l)
	{
		nr_fold = l;
		fprintf(stderr,"[rank %d] WARNING: # folds > # data. Will use # folds = # data instead (i.e., leave-one-out cross validation)\n",
		        mpi_get_rank());
	}

	// set_print_string_function(&print_null);

	fold_start = Malloc(int,nr_fold+1);
	for(i=0;i<l;i++) perm[i]=i;
	for(i=0;i<l;i++)
	{
		int j = i+rand()%(l-i);
		swap(perm[i],perm[j]);
	}
	for(i=0;i<=nr_fold;i++)
		fold_start[i]=i*l/nr_fold;

	for(i=0;i<nr_fold;i++)
	{
		int begin = fold_start[i];
		int end = fold_start[i+1];
		int j,k;
		struct problem subprob;

		subprob.bias = prob->bias;
		subprob.n = prob->n;
		subprob.l = end-begin;
		subprob.x = Malloc(struct feature_node*,subprob.l);
		subprob.y = Malloc(double,subprob.l);

		k=0;
		for(j=begin;j<end;j++)
		{
			subprob.x[k] = prob->x[perm[j]];
			subprob.y[k] = prob->y[perm[j]];
			++k;
		}

		param1.C = (param->C)*nr_fold;
		param1.init_sol = NULL;
		param1.solver_type = L1R_LR;
		param1.eps = 0.01;

		info("\nwarm start (%d/%d) begins!\n", i + 1, nr_fold);
		struct model *submodel = train(&subprob,&param1);

		int total_w_size;
		if(submodel->nr_class == 2)
			total_w_size = subprob.n;
		else
			total_w_size = subprob.n * submodel->nr_class;

		w_vectors[i] = Malloc(double, total_w_size);
		for(j=0; j<total_w_size; j++)
			w_vectors[i][j] = submodel->w[j];

		free_and_destroy_model(&submodel);
		free(subprob.x);
		free(subprob.y);
	}

	free(fold_start);
	free(perm);

	set_print_string_function(default_print_string);

	double *init_sol=Malloc(double, prob->n);
	for(int j=0; j<prob->n; j++)
		init_sol[j] = 0;
	for(i=0; i<nr_fold; i++)
		for(int j=0; j<prob->n; j++)
			init_sol[j] += w_vectors[i][j];
	for(int j=0; j<prob->n; j++)
		init_sol[j] /= (double) nr_fold;

	for(i=0; i<nr_fold; i++)
		free(w_vectors[i]);
	free(w_vectors);

	double eps = param->eps;
	int pos = 0;
	int neg = 0;
	for(int i=0;i<prob->l;i++)
		if(prob->y[i] > 0)
			pos++;
	neg = prob->l - pos;
	mpi_allreduce(&pos, 1, MPI_INT, MPI_SUM);
	mpi_allreduce(&neg, 1, MPI_INT, MPI_SUM);

	double primal_solver_tol = eps*max(min(pos,neg), 1)/prob->l;

	double *C = new double[prob->l];
	for(int i = 0; i < prob->l; i++)
		C[i] = param->C;

	problem prob_col;
	feature_node *x_space = NULL;
	transpose(prob, &x_space ,&prob_col);

	double *smallw = new double[prob_col.n];

	struct problem subprob_col;
	subprob_col.bias = prob_col.bias;
	subprob_col.l = l;
	subprob_col.x = Malloc(struct feature_node*,prob_col.n);
	subprob_col.y = Malloc(double,l);
	int wnz_num = 0;

	for(int j = 0; j < prob_col.n; j++)
	{
		if (init_sol[j] != 0)
		{
			subprob_col.x[wnz_num] = prob_col.x[j];

			smallw[wnz_num] = init_sol[j];
			++wnz_num;
		}
	}
	subprob_col.n = wnz_num;
	for(i=0;i<l;i++)
		subprob_col.y[i] = prob_col.y[i];

	info("combined # of nnz in w %d\n", wnz_num);

	l1r_lr_fun *l1r_fun_obj = new l1r_lr_fun_col(&subprob_col, C);
	OWLQN owlqn_obj_sub(l1r_fun_obj, primal_solver_tol, param->m, param->eta);
	owlqn_obj_sub.set_print_string(liblinear_print_string);
	owlqn_obj_sub.owlqn(smallw, param->solver_type);
	/*
	L1_LCOMM_STEP lcomm_step_obj_sub(l1r_fun_obj, primal_solver_tol, param->m, param->eta);
	lcomm_step_obj_sub.set_print_string(liblinear_print_string);
	lcomm_step_obj_sub.lcomm_step(smallw);
	*/
	delete l1r_fun_obj;

	free(subprob_col.x);
	free(subprob_col.y);

	double *w = new double[prob->n];

	wnz_num = 0;
	for(int j = 0; j < prob_col.n; j++)
	{
		if (init_sol[j] != 0)
		{
			w[j] = smallw[wnz_num];
			++wnz_num;
		}
		else
			w[j] = 0;
	}

	l1r_fun_obj = new l1r_lr_fun_col(&prob_col, C);
	OWLQN owlqn_obj(l1r_fun_obj, primal_solver_tol, param->m, param->eta);
	owlqn_obj.set_print_string(liblinear_print_string);
	owlqn_obj.owlqn(w, param->solver_type);
	delete l1r_fun_obj;
	delete[] C;
	delete [] prob_col.y;
	delete [] prob_col.x;
	delete [] x_space;

	delete [] smallw;
	delete [] w;
}

double predict_values(const struct model *model_, const struct feature_node *x, double *dec_values)
{
	int idx;
	int n;
	if(model_->bias>=0)
		n=model_->nr_feature+1;
	else
		n=model_->nr_feature;
	double *w=model_->w;
	int nr_class=model_->nr_class;
	int i;
	int nr_w;
	if(nr_class==2 && model_->param.solver_type != MCSVM_CS)
		nr_w = 1;
	else
		nr_w = nr_class;

	const feature_node *lx=x;
	for(i=0;i<nr_w;i++)
		dec_values[i] = 0;
	for(; (idx=lx->index)!=-1; lx++)
	{
		// the dimension of testing data may exceed that of training
		if(idx<=n)
			for(i=0;i<nr_w;i++)
				dec_values[i] += w[(idx-1)*nr_w+i]*lx->value;
	}

	if(nr_class==2)
	{
		if(check_regression_model(model_))
			return dec_values[0];
		else
			return (dec_values[0]>0)?model_->label[0]:model_->label[1];
	}
	else
	{
		int dec_max_idx = 0;
		for(i=1;i<nr_class;i++)
		{
			if(dec_values[i] > dec_values[dec_max_idx])
				dec_max_idx = i;
		}
		return model_->label[dec_max_idx];
	}
}

double predict(const model *model_, const feature_node *x)
{
	double *dec_values = Malloc(double, model_->nr_class);
	double label=predict_values(model_, x, dec_values);
	free(dec_values);
	return label;
}

double predict_probability(const struct model *model_, const struct feature_node *x, double* prob_estimates)
{
	if(check_probability_model(model_))
	{
		int i;
		int nr_class=model_->nr_class;
		int nr_w;
		if(nr_class==2)
			nr_w = 1;
		else
			nr_w = nr_class;

		double label=predict_values(model_, x, prob_estimates);
		for(i=0;i<nr_w;i++)
			prob_estimates[i]=1/(1+exp(-prob_estimates[i]));

		if(nr_class==2) // for binary classification
			prob_estimates[1]=1.-prob_estimates[0];
		else
		{
			double sum=0;
			for(i=0; i<nr_class; i++)
				sum+=prob_estimates[i];

			for(i=0; i<nr_class; i++)
				prob_estimates[i]=prob_estimates[i]/sum;
		}

		return label;
	}
	else
		return 0;
}

static const char *solver_type_table[]=
{
	// for multi-class classification
	"L2R_LR",
	"L2R_L2LOSS_SVC_DUAL",
	"L2R_L2LOSS_SVC",
	"L2R_L1LOSS_SVC_DUAL",
	"MCSVM_CS",
	"L1R_L2LOSS_SVC",
	"L1R_LR",
	"L2R_LR_DUAL",
	"L2R_LR_LBFGS",
	"L2R_LR_LCOMM_STEP",
	// for regression
	"L2R_L2LOSS_SVR",
	"L2R_L2LOSS_SVR_DUAL",
	"L2R_L1LOSS_SVR_DUAL",
	// for l1-regularized classification
	"L1R_LR_OWLQN",
	"L1R_LR_OWLQN_COL",
	"L1R_LR_OWLQN_VF",
	"L1R_LR_OWLQN_VF_COL",
	"L1R_LR_NEWTON",
	"L1R_LR_NEWTON_COL",
	"L1R_LR_PROJGRAD",
	"L1R_LR_PROJGRAD_COL",
	"L1R_LR_LCOMM_PG",
	"L1R_LR_LCOMM_PG_COL",
	"L1R_LR_LCOMM_STEP",
	"L1R_LR_LCOMM_STEP_COL",
	"L1R_LR_LCOMM_LBFGS",
	"L1R_LR_LCOMM_LBFGS_COL",
	NULL
};

int save_model(const char *model_file_name, const struct model *model_)
{
	int i;
	int nr_feature=model_->nr_feature;
	int n;
	const parameter& param = model_->param;

	if(model_->bias>=0)
		n=nr_feature+1;
	else
		n=nr_feature;
	int w_size = n;
	FILE *fp = fopen(model_file_name,"w");
	if(fp==NULL) return -1;

	char *old_locale = setlocale(LC_ALL, NULL);
	if (old_locale)
	{
		old_locale = strdup(old_locale);
	}
	setlocale(LC_ALL, "C");

	int nr_w;
	if(model_->nr_class==2 && model_->param.solver_type != MCSVM_CS)
		nr_w=1;
	else
		nr_w=model_->nr_class;

	fprintf(fp, "solver_type %s\n", solver_type_table[param.solver_type]);
	fprintf(fp, "nr_class %d\n", model_->nr_class);

	if(model_->label)
	{
		fprintf(fp, "label");
		for(i=0; i<model_->nr_class; i++)
			fprintf(fp, " %d", model_->label[i]);
		fprintf(fp, "\n");
	}

	fprintf(fp, "nr_feature %d\n", nr_feature);

	fprintf(fp, "bias %.16g\n", model_->bias);

	fprintf(fp, "w\n");
	for(i=0; i<w_size; i++)
	{
		int j;
		for(j=0; j<nr_w; j++)
			fprintf(fp, "%.16g ", model_->w[i*nr_w+j]);
		fprintf(fp, "\n");
	}

	setlocale(LC_ALL, old_locale);
	free(old_locale);

	if (ferror(fp) != 0 || fclose(fp) != 0) return -1;
	else return 0;
}

//
// FSCANF helps to handle fscanf failures.
// Its do-while block avoids the ambiguity when
// if (...)
//    FSCANF();
// is used
//
#define FSCANF(_stream, _format, _var)do\
{\
	if (fscanf(_stream, _format, _var) != 1)\
	{\
		fprintf(stderr, "ERROR: fscanf failed to read the model\n");\
		EXIT_LOAD_MODEL()\
	}\
}while(0)
// EXIT_LOAD_MODEL should NOT end with a semicolon.
#define EXIT_LOAD_MODEL()\
{\
	setlocale(LC_ALL, old_locale);\
	free(model_->label);\
	free(model_);\
	free(old_locale);\
	return NULL;\
}
struct model *load_model(const char *model_file_name)
{
	FILE *fp = fopen(model_file_name,"r");
	if(fp==NULL) return NULL;

	int i;
	int nr_feature;
	int n;
	int nr_class;
	double bias;
	model *model_ = Malloc(model,1);
	parameter& param = model_->param;

	model_->label = NULL;

	char *old_locale = setlocale(LC_ALL, NULL);
	if (old_locale)
	{
		old_locale = strdup(old_locale);
	}
	setlocale(LC_ALL, "C");

	char cmd[81];
	while(1)
	{
		FSCANF(fp,"%80s",cmd);
		if(strcmp(cmd,"solver_type")==0)
		{
			FSCANF(fp,"%80s",cmd);
			int i;
			for(i=0;solver_type_table[i];i++)
			{
				if(strcmp(solver_type_table[i],cmd)==0)
				{
					param.solver_type=i;
					break;
				}
			}
			if(solver_type_table[i] == NULL)
			{
				fprintf(stderr,"[rank %d] unknown solver type.\n", mpi_get_rank());
				EXIT_LOAD_MODEL()
			}
		}
		else if(strcmp(cmd,"nr_class")==0)
		{
			FSCANF(fp,"%d",&nr_class);
			model_->nr_class=nr_class;
		}
		else if(strcmp(cmd,"nr_feature")==0)
		{
			FSCANF(fp,"%d",&nr_feature);
			model_->nr_feature=nr_feature;
		}
		else if(strcmp(cmd,"bias")==0)
		{
			FSCANF(fp,"%lf",&bias);
			model_->bias=bias;
		}
		else if(strcmp(cmd,"w")==0)
		{
			break;
		}
		else if(strcmp(cmd,"label")==0)
		{
			int nr_class = model_->nr_class;
			model_->label = Malloc(int,nr_class);
			for(int i=0;i<nr_class;i++)
				FSCANF(fp,"%d",&model_->label[i]);
		}
		else
		{
			fprintf(stderr,"[rank %d] unknown text in model file: [%s]\n",mpi_get_rank(),cmd);
			EXIT_LOAD_MODEL()
		}
	}

	nr_feature=model_->nr_feature;
	if(model_->bias>=0)
		n=nr_feature+1;
	else
		n=nr_feature;
	int w_size = n;
	int nr_w;
	if(nr_class==2 && param.solver_type != MCSVM_CS)
		nr_w = 1;
	else
		nr_w = nr_class;

	model_->w=Malloc(double, w_size*nr_w);
	for(i=0; i<w_size; i++)
	{
		int j;
		for(j=0; j<nr_w; j++)
			FSCANF(fp, "%lf ", &model_->w[i*nr_w+j]);
	}

	setlocale(LC_ALL, old_locale);
	free(old_locale);

	if (ferror(fp) != 0 || fclose(fp) != 0) return NULL;

	return model_;
}

int get_nr_feature(const model *model_)
{
	return model_->nr_feature;
}

int get_nr_class(const model *model_)
{
	return model_->nr_class;
}

void get_labels(const model *model_, int* label)
{
	if (model_->label != NULL)
		for(int i=0;i<model_->nr_class;i++)
			label[i] = model_->label[i];
}

// use inline here for better performance (around 20% faster than the non-inline one)
static inline double get_w_value(const struct model *model_, int idx, int label_idx)
{
	int nr_class = model_->nr_class;
	int solver_type = model_->param.solver_type;
	const double *w = model_->w;

	if(idx < 0 || idx > model_->nr_feature)
		return 0;
	if(check_regression_model(model_))
		return w[idx];
	else
	{
		if(label_idx < 0 || label_idx >= nr_class)
			return 0;
		if(nr_class == 2 && solver_type != MCSVM_CS)
		{
			if(label_idx == 0)
				return w[idx];
			else
				return -w[idx];
		}
		else
			return w[idx*nr_class+label_idx];
	}
}

// feat_idx: starting from 1 to nr_feature
// label_idx: starting from 0 to nr_class-1 for classification models;
//            for regression models, label_idx is ignored.
double get_decfun_coef(const struct model *model_, int feat_idx, int label_idx)
{
	if(feat_idx > model_->nr_feature)
		return 0;
	return get_w_value(model_, feat_idx-1, label_idx);
}

double get_decfun_bias(const struct model *model_, int label_idx)
{
	int bias_idx = model_->nr_feature;
	double bias = model_->bias;
	if(bias <= 0)
		return 0;
	else
		return bias*get_w_value(model_, bias_idx, label_idx);
}

void free_model_content(struct model *model_ptr)
{
	if(model_ptr->w != NULL)
		free(model_ptr->w);
	if(model_ptr->label != NULL)
		free(model_ptr->label);
}

void free_and_destroy_model(struct model **model_ptr_ptr)
{
	struct model *model_ptr = *model_ptr_ptr;
	if(model_ptr != NULL)
	{
		free_model_content(model_ptr);
		free(model_ptr);
	}
}

void destroy_param(parameter* param)
{
	if(param->weight_label != NULL)
		free(param->weight_label);
	if(param->weight != NULL)
		free(param->weight);
	if(param->init_sol != NULL)
		free(param->init_sol);
}

const char *check_parameter(const problem *prob, const parameter *param)
{
	if(param->eps <= 0)
		return "eps <= 0";

	if(param->C <= 0)
		return "C <= 0";

	if(param->p < 0)
		return "p < 0";

	if(param->solver_type != L2R_LR
		&& param->solver_type != L2R_L2LOSS_SVC_DUAL
		&& param->solver_type != L2R_L2LOSS_SVC
		&& param->solver_type != L2R_L1LOSS_SVC_DUAL
		&& param->solver_type != MCSVM_CS
		&& param->solver_type != L1R_L2LOSS_SVC
		&& param->solver_type != L1R_LR
		&& param->solver_type != L2R_LR_DUAL
		&& param->solver_type != L2R_LR_LBFGS
		&& param->solver_type != L2R_L2LOSS_SVR
		&& param->solver_type != L2R_L2LOSS_SVR_DUAL
		&& param->solver_type != L2R_L1LOSS_SVR_DUAL
		&& param->solver_type != L1R_LR_OWLQN
		&& param->solver_type != L1R_LR_OWLQN_COL
		&& param->solver_type != L1R_LR_OWLQN_VF
		&& param->solver_type != L1R_LR_OWLQN_VF_COL
		&& param->solver_type != L1R_LR_NEWTON
		&& param->solver_type != L1R_LR_NEWTON_COL
		&& param->solver_type != L1R_LR_PROJGRAD
		&& param->solver_type != L1R_LR_PROJGRAD_COL
		&& param->solver_type != L1R_LR_LCOMM_PG
		&& param->solver_type != L1R_LR_LCOMM_PG_COL
		&& param->solver_type != L1R_LR_LCOMM_STEP
		&& param->solver_type != L1R_LR_LCOMM_STEP_COL
		&& param->solver_type != L1R_LR_LCOMM_LBFGS
		&& param->solver_type != L1R_LR_LCOMM_LBFGS_COL
	   )
		return "unknown solver type";

	if(param->init_sol != NULL
		&& param->solver_type != L2R_LR
		&& param->solver_type != L2R_L2LOSS_SVC
		&& param->solver_type != L2R_LR_LBFGS
		&& param->solver_type != L1R_L2LOSS_SVC
		&& param->solver_type != L1R_LR_OWLQN
		&& param->solver_type != L1R_LR_OWLQN_COL
		&& param->solver_type != L1R_LR_OWLQN_VF
		&& param->solver_type != L1R_LR_OWLQN_VF_COL
		&& param->solver_type != L1R_LR_NEWTON
		&& param->solver_type != L1R_LR_NEWTON_COL
		&& param->solver_type != L1R_LR_PROJGRAD
		&& param->solver_type != L1R_LR_PROJGRAD_COL
		&& param->solver_type != L1R_LR_LCOMM_PG
		&& param->solver_type != L1R_LR_LCOMM_PG_COL
		&& param->solver_type != L1R_LR_LCOMM_STEP
		&& param->solver_type != L1R_LR_LCOMM_STEP_COL
		&& param->solver_type != L1R_LR_LCOMM_LBFGS
		&& param->solver_type != L1R_LR_LCOMM_LBFGS_COL
	   )
		return "Initial-solution specification supported only for solver L2R_LR,  L2R_L2LOSS_SVC, and L2R_LR_LBFGS";

	return NULL;
}

int check_probability_model(const struct model *model_)
{
	return (model_->param.solver_type==L2R_LR ||
			model_->param.solver_type==L2R_LR_DUAL ||
			model_->param.solver_type==L1R_LR);
}

int check_regression_model(const struct model *model_)
{
	return (model_->param.solver_type==L2R_L2LOSS_SVR ||
			model_->param.solver_type==L2R_L1LOSS_SVR_DUAL ||
			model_->param.solver_type==L2R_L2LOSS_SVR_DUAL);
}

void set_print_string_function(void (*print_func)(const char*))
{
	if (print_func == NULL)
		liblinear_print_string = &print_string_stdout;
	else
		liblinear_print_string = print_func;
}

int mpi_get_rank()
{
	int rank;
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	return rank;	
}

int mpi_get_size()
{
	int size;
	MPI_Comm_size(MPI_COMM_WORLD, &size);
	return size;	
}

void mpi_exit(const int status)
{
	MPI_Finalize();
	exit(status);
}
